from pycaption.scc.translator import translate_scc


class TestSCCTranslator:

    def test_successful_translation(
            self, sample_scc_pop_on, sample_translated_scc_success):
        result = translate_scc(sample_scc_pop_on)

        assert sample_translated_scc_success == result

    def test_no_brackets(
            self, sample_scc_pop_on, sample_translated_scc_no_brackets):
        result = translate_scc(sample_scc_pop_on, brackets=None)

        assert sample_translated_scc_no_brackets == result

    def test_custom_brackets(
            self, sample_scc_pop_on, sample_translated_scc_custom_brackets):
        result = translate_scc(sample_scc_pop_on, brackets="{}")

        assert sample_translated_scc_custom_brackets == result

    def test_commands_not_found(self, sample_scc_with_unknown_commands,
                                sample_translated_scc_commands_not_found):
        result = translate_scc(sample_scc_with_unknown_commands)

        assert sample_translated_scc_commands_not_found == result



import pytest

from pycaption import SCCReader, CaptionReadNoCaptions, CaptionNode
from pycaption.geometry import (
    UnitEnum, HorizontalAlignmentEnum, VerticalAlignmentEnum,
)
from pycaption.scc.specialized_collections import (
    InstructionNodeCreator, TimingCorrectingCaptionList,
)
from pycaption.scc.state_machines import DefaultProvidingPositionTracker
from tests.mixins import ReaderTestingMixIn

TOLERANCE_MICROSECONDS = 500 * 1000


class TestSCCReader(ReaderTestingMixIn):
    def setup_method(self):
        self.reader = SCCReader()

    def test_positive_answer_for_detection(self, sample_scc_pop_on):
        super().assert_positive_answer_for_detection(sample_scc_pop_on)

    @pytest.mark.parametrize('different_sample', [
        pytest.lazy_fixture('sample_dfxp'),
        pytest.lazy_fixture('sample_microdvd'),
        pytest.lazy_fixture('sample_sami'),
        pytest.lazy_fixture('sample_srt'),
        pytest.lazy_fixture('sample_webvtt')
    ])
    def test_negative_answer_for_detection(self, different_sample):
        super().assert_negative_answer_for_detection(different_sample)

    def test_caption_length(self, sample_scc_pop_on):
        captions = SCCReader().read(sample_scc_pop_on)

        assert 7 == len(captions.get_captions("en-US"))

    def test_proper_timestamps(self, sample_scc_pop_on):
        captions = SCCReader().read(sample_scc_pop_on)
        paragraph = captions.get_captions("en-US")[2]

        delta_start = abs(paragraph.start - 17000000)
        delta_end = abs(paragraph.end - 18752000)

        assert delta_start < TOLERANCE_MICROSECONDS
        assert delta_end < TOLERANCE_MICROSECONDS

    def test_empty_file(self, sample_scc_empty):
        with pytest.raises(CaptionReadNoCaptions):
            SCCReader().read(sample_scc_empty)

    def test_positioning(self, sample_scc_multiple_positioning):
        captions = SCCReader().read(sample_scc_multiple_positioning)

        # SCC generates only origin, and we always expect it.
        expected_positioning = [
            ((10.0, UnitEnum.PERCENT), (77.0, UnitEnum.PERCENT)),
            ((40.0, UnitEnum.PERCENT), (5.0, UnitEnum.PERCENT)),
            ((70.0, UnitEnum.PERCENT), (23.0, UnitEnum.PERCENT)),
            ((20.0, UnitEnum.PERCENT), (47.0, UnitEnum.PERCENT)),
            ((20.0, UnitEnum.PERCENT), (89.0, UnitEnum.PERCENT)),
            ((40.0, UnitEnum.PERCENT), (53.0, UnitEnum.PERCENT)),
            ((70.0, UnitEnum.PERCENT), (17.0, UnitEnum.PERCENT)),
            ((20.0, UnitEnum.PERCENT), (35.0, UnitEnum.PERCENT)),
            ((20.0, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
            ((70.0, UnitEnum.PERCENT), (11.0, UnitEnum.PERCENT)),
            ((40.0, UnitEnum.PERCENT), (41.0, UnitEnum.PERCENT)),
            ((20.0, UnitEnum.PERCENT), (71.0, UnitEnum.PERCENT))
        ]

        actual_positioning = [
            caption_.layout_info.origin.serialized()
            for caption_ in captions.get_captions('en-US')
        ]

        assert expected_positioning == actual_positioning

    def test_tab_offset(self, sample_scc_tab_offset):
        captions = SCCReader().read(sample_scc_tab_offset)

        # SCC generates only origin, and we always expect it.
        expected_positioning = [
            ((37.5, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
            ((17.5, UnitEnum.PERCENT), (89.0, UnitEnum.PERCENT)),
            ((12.5, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
            ((27.5, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
            ((30.0, UnitEnum.PERCENT), (89.0, UnitEnum.PERCENT)),
            ((35.0, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
            ((17.5, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT))
        ]

        actual_positioning = [
            caption_.layout_info.origin.serialized()
            for caption_ in captions.get_captions('en-US')
        ]

        assert expected_positioning == actual_positioning

    def test_italics_are_properly_read(self, sample_scc_with_italics):
        def switches_italics(node):
            """Determine if the current node switches italics on or off, or
            raise ValueError is it's not a style node

            Style nodes should be deprecated in favor of another model, so this
            function is expected to go away.

            :type node: pycaption.CaptionNode
            :rtype: bool
            """
            if not node.type_ == node.STYLE:
                raise ValueError("This should be a style node.")

            return node.start

        caption_set = SCCReader().read(sample_scc_with_italics)
        nodes = caption_set.get_captions('en-US')[0].nodes

        # We assert that the text is specified in italics.
        # If Style nodes are replaced, the way these 3 assertions are made
        # will most likely change
        assert switches_italics(nodes[0]) is True
        assert switches_italics(nodes[2]) is False
        assert nodes[1].content == 'abababab'

    def test_default_positioning_when_no_positioning_is_specified(
            self, sample_no_positioning_at_all_scc):
        caption_set = SCCReader().read(sample_no_positioning_at_all_scc)

        actual_caption_layouts = [
            caption.layout_info.serialized()
            for caption in caption_set.get_captions('en-US')
        ]

        expected_caption_layouts = [
            (((10.0, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
             None, None,
             (HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP)),
            (((10.0, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
             None, None,
             (HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP))
        ]

        actual_node_layout_infos = [
            {idx: [node.layout_info.serialized() for node in caption.nodes]}
            for idx, caption in enumerate(caption_set.get_captions('en-US'))
        ]

        expected_node_layout_infos = [
            {0: [(
                ((10.0, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
                None, None,
                (HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP)
            )]},
            {1: [(
                ((10.0, UnitEnum.PERCENT), (83.0, UnitEnum.PERCENT)),
                None, None,
                (HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP)
            )]}
        ]

        assert expected_node_layout_infos == actual_node_layout_infos
        assert expected_caption_layouts == actual_caption_layouts

    def test_timing_is_properly_set_on_split_captions(
            self, sample_scc_produces_captions_with_start_and_end_time_the_same
    ):
        caption_set = SCCReader().read(
            sample_scc_produces_captions_with_start_and_end_time_the_same
        )
        expected_timings = [
            ('00:01:35.666', '00:01:40.866'),
            ('00:01:35.666', '00:01:40.866'),
            ('00:01:35.666', '00:01:40.866'),
        ]

        actual_timings = [
            (c_.format_start(), c_.format_end())
            for c_ in caption_set.get_captions('en-US')
        ]

        assert expected_timings == actual_timings

    def test_removed_extended_characters_ascii_duplicate(
            self, sample_scc_with_extended_characters):
        caption_set = SCCReader().read(sample_scc_with_extended_characters)
        nodes = caption_set.get_captions('en-US')[0].nodes

        assert nodes[0].content == 'MÄRTHA:'

    def test_ignore_repeated_tab_offset(self, sample_scc_repeated_tab_offset):
        expected_lines = [
            '[Radio reporter]',
            'The I-10 Santa Monica Freeway',
            'westbound is jammed,',
            'due to a three-car accident',
            'blocking lanes 1 and 2',
        ]

        caption_set = SCCReader().read(sample_scc_repeated_tab_offset)
        actual_lines = [
            node.content
            for cap_ in caption_set.get_captions('en-US')
            for node in cap_.nodes
            if node.type_ == CaptionNode.TEXT
        ]

        assert expected_lines == actual_lines


class TestCoverageOnly:
    """In order to refactor safely, we need coverage of 95% or more.
     This class includes tests that ensure that at the very least, we don't
     break anything that was working, OR fix anything whose faulty behavior
      was accepted.

      All the tests in this suite should only be useful for refactoring. They
      DO NOT ensure functionality. They only ensure nothing changes.
    """

    def test_freeze_rollup_captions_contents(self, sample_scc_roll_up_ru2):
        # There were no tests for ROLL-UP captions, but the library processed
        # Roll-Up captions. Make sure nothing changes during the refactoring
        scc1 = SCCReader().read(sample_scc_roll_up_ru2)
        captions = scc1.get_captions('en-US')
        actual_texts = [cap_.nodes[0].content for cap_ in captions]
        expected_texts = [
            '>>> HI.',
            "I'M KEVIN CUNNING AND AT",
            "INVESTOR'S BANK WE BELIEVE IN",
            'HELPING THE LOCAL NEIGHBORHOODS',
            'AND IMPROVING THE LIVES OF ALL',
            'WE SERVE.',
            '®°½½',
            'ABû',
            'ÁÁÉÓ¡',
            "WHERE YOU'RE STANDING NOW,",
            "LOOKING OUT THERE, THAT'S AL",
            'THE CROWD.',
            '>> IT WAS GOOD TO BE IN TH',
            "And restore Iowa's land, water",
            'And wildlife.',
            '>> Bike Iowa, your source for',
        ]

        assert expected_texts == actual_texts

    def test_multiple_formats(self, sample_scc_multiple_formats):
        # Test for captions that contain both pop on and paint on formats to
        # ensure the paint on lines are not repeated
        expected_text_lines = [
            "(Client's Voice)",
            'Remember that degree',
            'you got in taxation?',
            '(Danny)',
            "Of course you don't",
            "because you didn't!",
            "Your job isn't doing hard",
            'work...',
            "...it's making them do hard",
            'work...',
            '...and getting paid for it.',
            '(VO)',
            'Snap and sort your expenses to',
            'save over $4,600 at tax time.',
            'QUICKBOOKS. BACKING YOU.',
        ]

        captions = SCCReader().read(sample_scc_multiple_formats) \
            .get_captions('en-US')
        text_lines = [
            node.content
            for caption in captions
            for node in caption.nodes
            if node.type_ == CaptionNode.TEXT
        ]

        assert expected_text_lines == text_lines

    def test_freeze_semicolon_spec_time(self, sample_scc_roll_up_ru2):
        scc1 = SCCReader().read(sample_scc_roll_up_ru2)
        captions = scc1.get_captions('en-US')
        expected_timings = [
            (766666.6666666667, 2800000.0),
            (2800000.0, 4600000.0),
            (4600000.0, 6166666.666666667),
            (6166666.666666667, 9733333.333333332),
            (9733333.333333332, 11266666.666666668),
            (11266666.666666668, 12266666.666666668),
            (12266666.666666668, 13266666.666666668),
            (13266666.666666668, 14266666.666666668),
            (14266666.666666668, 17066666.666666668),
            (17066666.666666668, 18666666.666666668),
            (18666666.666666668, 20233333.333333336),
            (20233333.333333336, 21833333.333333332),
            (21833333.333333332, 34933333.33333333),
            (34933333.33333333, 36433333.33333333),
            (36433333.33333333, 44300000.0),
            (44300000.0, 44866666.666666664),
        ]

        actual_timings = [(c_.start, c_.end) for c_ in captions]

        assert expected_timings == actual_timings

    def test_freeze_colon_spec_time(self, sample_scc_pop_on):
        # Coverage doesn't mean we test that functionality, so assert that
        # all the timing specs that previously had coverage, will actually
        # remain unchanged.
        scc1 = SCCReader().read(sample_scc_pop_on)
        expected_timings = [
            (9776433.333333332, 12312300.0),
            (14781433.33333333, 16883533.333333332),
            (16950266.666666664, 18618600.000000004),
            (18685333.333333332, 20754066.666666664),
            (20820800.0, 26626600.0),
            (26693333.333333332, 32098733.333333332),
            (32165466.66666666, 36202833.33333332),
        ]

        actual_timings = [
            (c_.start, c_.end) for c_ in scc1.get_captions('en-US')]

        assert expected_timings == actual_timings


class TestInterpretableNodeCreator:
    def test_italics_commands_are_formatted_properly(self):
        node_creator = InstructionNodeCreator(
            position_tracker=(DefaultProvidingPositionTracker())
        )

        # We expect
        # 1. all the initial italics closing nodes to be removed
        # 2. all the redundant italic nodes to be trimmed
        # 3. to get new closing italic nodes before changing position,
        # 4. to get new opening italic nodes after changing position, if 3
        # happened
        # 5. to get a final italic closing node, if one is needed
        node_creator.interpret_command('9470')  # row 15, col 0
        node_creator.interpret_command('9120')  # italics off
        node_creator.interpret_command('9120')  # italics off
        node_creator.add_chars('a')

        node_creator.interpret_command('9770')  # row 10 col 0
        node_creator.interpret_command('91ae')  # italics ON
        node_creator.add_chars('b')
        node_creator.interpret_command('91ae')  # italics ON
        node_creator.interpret_command('91ae')  # italics ON
        node_creator.interpret_command('9120')  # italics OFF
        node_creator.interpret_command('9120')  # italics OFF
        node_creator.interpret_command('91ae')  # italics ON
        node_creator.interpret_command('91ae')  # italics ON
        node_creator.interpret_command('91ae')  # italics ON
        node_creator.add_chars('b')
        node_creator.interpret_command('91ae')  # italics ON again
        node_creator.add_chars('b')
        node_creator.interpret_command('9120')  # italics OFF
        node_creator.interpret_command('9120')  # italics OFF

        node_creator.interpret_command('1570')  # row 6 col 0
        node_creator.add_chars('c')
        node_creator.interpret_command('91ae')  # italics ON

        node_creator.interpret_command('9270')  # row 4 col 0
        node_creator.add_chars('d')

        node_creator.interpret_command('15d0')  # row 5 col 0 - creates BR
        node_creator.add_chars('e')

        node_creator.interpret_command('1570')  # row 6 col 0 - creates BR
        node_creator.add_chars('f')

        result = list(node_creator)

        assert result[0].is_text_node()
        assert result[1].requires_repositioning()
        assert result[2].is_italics_node()
        assert result[2].sets_italics_on()

        assert result[3].is_text_node()
        assert result[4].is_text_node()
        assert result[5].is_text_node()

        assert result[6].is_italics_node()
        assert result[6].sets_italics_off()

        assert result[7].requires_repositioning()
        assert result[8].is_text_node()

        assert result[9].requires_repositioning()
        assert result[10].is_italics_node()
        assert result[10].sets_italics_on()

        assert result[11].is_text_node()
        assert result[12].is_explicit_break()
        assert result[13].is_text_node()
        assert result[14].is_explicit_break()
        assert result[15].is_text_node()

        assert result[16].is_italics_node()
        assert result[16].sets_italics_off()


class CaptionDummy:
    """Mock for pycaption.base.Caption"""

    def __init__(self, start=0, end=0, nodes=(1, 2)):
        self.nodes = nodes
        self.start = start
        self.end = end

    def __repr__(self):
        return f'{self.start}-->{self.end}'


class TestTimingCorrectingCaptionList:
    def test_appending_then_appending(self):
        caption_list = TimingCorrectingCaptionList()

        caption_list.append(CaptionDummy(start=3))
        caption_list.append(CaptionDummy(start=6))

        assert caption_list[0].end == 6

    def test_appending_then_extending(self):
        caption_list = TimingCorrectingCaptionList()

        caption_list.append(CaptionDummy(start=3))
        caption_list.extend([CaptionDummy(start=7), CaptionDummy(start=7)])

        assert caption_list[0].end == 7

    def test_extending_then_appending(self):
        caption_list = TimingCorrectingCaptionList()

        caption_list.extend([CaptionDummy(start=4), CaptionDummy(start=4)])
        caption_list.append(CaptionDummy(start=9))

        assert caption_list[0].end == 9
        assert caption_list[1].end == 9

    def test_extending_then_extending(self):
        caption_list = TimingCorrectingCaptionList()

        caption_list.extend([CaptionDummy(start=4), CaptionDummy(start=4)])
        caption_list.extend([CaptionDummy(start=7), CaptionDummy(start=7)])

        assert caption_list[0].end == 7
        assert caption_list[1].end == 7

    def test_not_appending_none_or_empty_captions(self):
        caption_list = TimingCorrectingCaptionList()

        caption_list.append(None)
        caption_list.extend([CaptionDummy(nodes=[])])

        assert len(caption_list) == 0

    def test_not_extending_list_with_nones_or_empty_captions(self):
        caption_list = TimingCorrectingCaptionList()

        caption_list.extend([None, CaptionDummy(nodes=[])])

        assert len(caption_list) == 0

    def test_not_overwriting_end_time(self):
        # Test here all the 4 cases:
        caption_list = TimingCorrectingCaptionList()

        caption_list.append(CaptionDummy(start=1, end=3))
        caption_list.append(CaptionDummy(start=5, end=6))

        # Append then append
        assert caption_list[-2].end == 3

        caption_list.extend([CaptionDummy(start=7, end=8)])

        # Append then extend
        assert caption_list[-2].end == 6

        caption_list.extend([CaptionDummy(start=9, end=10)])

        # extend then extend
        assert caption_list[-2].end == 8

        caption_list.append(CaptionDummy(start=11, end=12))

        # extend then append
        assert caption_list[-2].end == 10

    def test_last_caption_zero_end_time_is_corrected(
            self, sample_scc_no_explicit_end_to_last_caption):
        caption_set = SCCReader().read(
            sample_scc_no_explicit_end_to_last_caption
        )

        last_caption = caption_set.get_captions('en-US')[-1]

        assert last_caption.end == last_caption.start + 4 * 1000 * 1000

    def test_eoc_first_command(self, sample_scc_eoc_first_command):
        # TODO First caption should be ignored because it doesn't start with
        #  a pop/roll/paint on command
        caption_set = SCCReader().read(sample_scc_eoc_first_command)

        # just one caption, first EOC disappears
        num_captions = len(caption_set.get_captions('en-US'))

        assert num_captions == 2



import pytest


@pytest.fixture(scope="session")
def sample_microdvd():
    return """{230}{307}( clock ticking )
{371}{425}MAN:|When we think|\u266a ...say bow, wow, \u266a
{425}{468}we have this vision of Einstein
{468}{522}as an old, wrinkly man|with white hair.
{522}{669}MAN 2:|E equals m c-squared is|not about an old Einstein.
{669}{805}MAN 2:|It's all about an eternal Einstein.
{805}{905}<LAUGHING & WHOOPS!>
"""


@pytest.fixture(scope="session")
def sample_microdvd_2():
    return """{230}{307}( clock ticking )
{371}{425}MAN:|When we think|\u266a ...say bow, wow, \u266a
{425}{468}we have this vision of Einstein
{468}{522}|as an old, wrinkly man|with white hair.
{522}{669}MAN 2:|E equals m c-squared is|not about an old Einstein.
{669}{805}MAN 2:|It's all about an eternal Einstein.
{805}{905}<LAUGHING & WHOOPS!>
"""


@pytest.fixture(scope="session")
def sample_microdvd_invalid_format():
    return """{230}{307}( clock ticking )
{}{425}{567} MAN:|When we think|\u266a ...say bow, wow, \u266a
{425}{468}we have this vision of Einstein
"""


@pytest.fixture(scope="session")
def missing_fps_sample_microdvd():
    return """{301}{307}( clock ticking )
{0}{0} MAN:|When we think|\u266a ...say bow, wow, \u266a
"""


@pytest.fixture(scope="session")
def sample_microdvd_empty():
    return """
"""


@pytest.fixture(scope="session")
def sample_microdvd_empty_cue_output():
    return """{30}{57}abc"""



import os

try:
    import nltk.data
except ImportError:
    raise ImportError(
        'You must install nltk==2.0.4 and numpy==1.7.1 to be able to use this.')
from pycaption.base import BaseWriter, CaptionNode


class TranscriptWriter(BaseWriter):
    def __init__(self, *args, **kw):
        self.nltk = nltk.data.load(f'file:{os.path.dirname(__file__)}'
                                   '/english.pickle')

    def write(self, captions):
        transcripts = []

        for lang in captions.get_languages():
            lang_transcript = f'* {lang.upper()} Transcript *\n'

            for caption in captions.get_captions(lang):
                lang_transcript = self._strip_text(caption.nodes,
                                                   lang_transcript)

            lang_transcript = '\n'.join(self.nltk.tokenize(lang_transcript))
            transcripts.append(lang_transcript)

        return '\n'.join(transcripts)

    def _strip_text(self, elements, lang_transcript):
        return ' '.join([lang_transcript] + [el.content for el in elements if
                                             el.type_ == CaptionNode.TEXT])



import pytest


@pytest.fixture(scope="session")
def sample_webvtt():
    return """WEBVTT

00:09.209 --> 00:12.312
( clock ticking )

00:14.848 --> 00:17.000
MAN:
When we think
♪ ...say bow, wow, ♪

00:17.000 --> 00:18.752
we have this vision of Einstein

00:18.752 --> 00:20.887
as an old, wrinkly man
with white hair.

00:20.887 --> 00:26.760
MAN 2:
E equals m c-squared is
not about an old Einstein.

00:26.760 --> 00:32.200
MAN 2:
It's all about an eternal Einstein.

00:32.200 --> 00:36.200
<LAUGHING & WHOOPS!>
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_dfxp():
    return """WEBVTT

00:09.209 --> 00:12.312
( clock ticking )

00:14.848 --> 00:17.000
MAN:
When we think
♪ ...say bow, wow, ♪

00:17.000 --> 00:18.752 align:right
we have this vision of Einstein

00:18.752 --> 00:20.887
&nbsp;
as an old, wrinkly man
with white hair.

00:20.887 --> 00:26.760
MAN 2:
E equals m c-squared is
not about an old Einstein.

00:26.760 --> 00:32.200
MAN 2:
It's all about an eternal Einstein.

00:32.200 --> 00:36.200
&lt;LAUGHING &amp; WHOOPS!>
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_sami(sample_webvtt_from_dfxp):
    return sample_webvtt_from_dfxp


@pytest.fixture(scope="session")
def sample_webvtt_from_sami_with_style():
    return """WEBVTT

00:09.209 --> 00:12.312
I <b>do</b> <i>not</i> want to go <u>home</u>.
I don't like it <i><u><b>there</b></u></i>.
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_sami_with_id_style():
    return """WEBVTT

00:09.209 --> 00:12.312
<i>This is in italics.</i>

00:14.848 --> 00:17.000
<u>This is underlined.</u>

00:17.000 --> 00:18.752
<b>This is bold.</b>

00:20.887 --> 00:26.760
<b><i><u>This is everything together.</u></i></b>
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_dfxp_with_style():
    return """WEBVTT

00:09.209 --> 00:12.312
This is <i>italic</i>, <b>bold</b>, <u>underline</u>, <i><u><b>everything together in one tag</b></u></i>, and <u><b><i>nested</i></b></u>.
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_dfxp_with_positioning():
    return """WEBVTT

00:01.000 --> 00:03.000 position:25% line:25% size:50%
You might not remember us. We are a typical transparent region with centered text that has an outline.

00:03.500 --> 00:05.000 align:right position:25% line:25% size:50%
had personality.

00:05.500 --> 00:07.000 align:left position:50% line:50% size:25%
Hello there, children! Have you seen any visitors?

00:07.500 --> 00:09.000 align:right position:25% line:75% size:25%
This is
the last cue
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_dfxp_with_positioning_and_style():
    return """WEBVTT

00:01.000 --> 00:03.000 position:25% line:25% size:50%
You might not remember us. We are a typical transparent region with centered text that has an outline.

00:03.500 --> 00:05.000 align:right position:25% line:25% size:50%
had <u>personality.</u>

00:05.500 --> 00:07.000 align:left position:50% line:50% size:25%
Hello there, children! Have you seen any visitors?

00:07.500 --> 00:09.000 align:right position:25% line:75% size:25%
This is
the last cue
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_srt():
    return """WEBVTT

00:09.209 --> 00:12.312
( clock ticking )

00:14.848 --> 00:17.000
MAN:
When we think
♪ ...say bow, wow, ♪

00:17.000 --> 00:18.752
we have this vision of Einstein

00:18.752 --> 00:20.887
as an old, wrinkly man
with white hair.

00:20.887 --> 00:26.760
MAN 2:
E equals m c-squared is
not about an old Einstein.

00:26.760 --> 00:32.200
MAN 2:
It's all about an eternal Einstein.

00:32.200 --> 00:36.200
&lt;LAUGHING &amp; WHOOPS!>
"""


# This is not equal to the input because we accept unescaped illegal characters
# when reading (because many players do so) but escape them when writing
# in order to conform to the specification.
@pytest.fixture(scope="session")
def sample_webvtt_from_webvtt(sample_webvtt_from_srt):
    return sample_webvtt_from_srt


@pytest.fixture(scope="session")
def sample_webvtt_2():
    return """WEBVTT

1
00:00:00.000 --> 00:00:43.000
- HELLO WORLD!

2
00:00:59.000 --> 00:01:30.000
- LOOKING GOOOOD.

3
00:01:40.000 --> 00:02:00.000
- HA HA HA!

4
00:02:05.105 --> 00:03:07.007
- HI. WELCOME TO SESAME STREET.

5
00:04:07.007 --> 00:05:38.441
ON TONIGHT'S SHOW...

6
00:05:58.441 --> 00:06:40.543
- I'M NOT GOING TO WATCH THIS.

7
00:07:10.543 --> 00:07:51.711
HEY. WATCH THIS.
"""


@pytest.fixture(scope="session")
def sample_webvtt_empty():
    return """WEBVTT
"""


@pytest.fixture(scope="session")
def sample_webvtt_double_br():
    return """WEBVTT

00:14.848 --> 00:18.848
MAN:
&nbsp;
When we think
of "E equals m c-squared",
"""


@pytest.fixture(scope="session")
def sample_webvtt_output_long_cue():
    return """\
WEBVTT

00:01.000 --> 00:02.000
NARRATOR:

00:02.000 --> 00:03.000 position:25% line:25% size:65%
They built the largest, most incredible, wildest, craziest,

00:03.000 --> 00:04.000
most complex machine in history.
"""


@pytest.fixture(scope="session")
def webvtt_from_dfxp_with_conflicting_align():
    return """WEBVTT

00:04.537 --> 00:07.841
IT'S WORD GIRL♫

00:08.537 --> 00:10.841
♫WORD UP,
IT'S WORD GIRL♫
"""


@pytest.fixture(scope="session")
def sample_webvtt_with_cue_settings():
    return """\
WEBVTT

00:01.000 --> 00:06.000 align:middle position:37% line:74%
37% 74% - NARRATOR:

00:01.000 --> 00:06.000 this is invalid, but will also be kept
They built the largest,
"""


@pytest.fixture(scope="session")
def sample_webvtt_from_scc_properly_writes_newlines_output():
    return """\
WEBVTT

21:30.033 --> 21:34.033 align:left position:20% line:83% size:70%
aa
bb
"""


@pytest.fixture(scope="session")
def sample_webvtt_last_cue_zero_start():
    return """WEBVTT

00:00.000 --> 00:12.312
( clock ticking )"""


@pytest.fixture(scope="session")
def sample_webvtt_empty_cue():
    return """WEBVTT

1
00:00.000 --> 00:02.000

00:04.000 --> 00:05.000
Transcribed by Celestials
"""


@pytest.fixture(scope="session")
def sample_webvtt_multi_lang_en():
    return """WEBVTT

00:14.848 --> 00:18.848
Butterfly.
"""


@pytest.fixture(scope="session")
def sample_webvtt_multi_lang_de():
    return """WEBVTT

00:14.848 --> 00:18.848
Schmetterling.
"""


@pytest.fixture(scope="session")
def sample_webvtt_empty_cue_output():
    return """\
WEBVTT

00:01.209 --> 00:02.312 position:10% line:10% size:80%
abc
"""


@pytest.fixture(scope="session")
def sample_webvtt_timestamps():
    return """WEBVTT
    
01:01.001 --> 10:10.100
Test zero padded and two digit timestamps without hours

01:01:01.001 --> 10:10:10.100
Test zero padded and two digit timestamps without hours"""



import pytest

from pycaption import SRTReader, CaptionReadNoCaptions
from tests.mixins import ReaderTestingMixIn


class TestSRTReader(ReaderTestingMixIn):
    def setup_class(self):
        self.reader = SRTReader()

    def test_positive_answer_for_detection(self, sample_srt):
        super().assert_positive_answer_for_detection(sample_srt)

    @pytest.mark.parametrize('different_sample', [
        pytest.lazy_fixture('sample_dfxp'),
        pytest.lazy_fixture('sample_microdvd'),
        pytest.lazy_fixture('sample_sami'),
        pytest.lazy_fixture('sample_scc_pop_on'),
        pytest.lazy_fixture('sample_webvtt')
    ])
    def test_negative_answer_for_detection(self, different_sample):
        super().assert_negative_answer_for_detection(different_sample)

    def test_caption_length(self, sample_srt):
        captions = self.reader.read(sample_srt)

        assert 7 == len(captions.get_captions("en-US"))

    def test_proper_timestamps(self, sample_srt):
        captions = self.reader.read(sample_srt)
        third_paragraph = captions.get_captions("en-US")[2]

        assert 17000000 == third_paragraph.start
        assert 18752000 == third_paragraph.end

    def test_numeric_captions(self, sample_srt_numeric):
        captions = self.reader.read(sample_srt_numeric)
        paragraphs = captions.get_captions("en-US")

        assert 7 == len(captions.get_captions("en-US"))
        assert paragraphs[-3].get_text() == "NUMBER  IS  662-429-84-77."
        assert paragraphs[-1].get_text() == "3"

    def test_empty_file(self, sample_srt_empty):
        with pytest.raises(CaptionReadNoCaptions) as exc_info:
            self.reader.read(sample_srt_empty)
        assert exc_info.value.args[0] == 'empty caption file'

    def test_extra_empty_line(self, sample_srt_blank_lines):
        captions = self.reader.read(sample_srt_blank_lines)
        paragraphs = captions.get_captions("en-US")

        assert 2 == len(paragraphs)
        assert '\n' not in paragraphs[0].get_text()
        assert '\n' not in paragraphs[1].get_text()

    def test_extra_trailing_empty_line(self, sample_srt_trailing_blanks):
        captions = self.reader.read(sample_srt_trailing_blanks)
        paragraphs = captions.get_captions("en-US")

        assert 2 == len(paragraphs)
        assert '\n' not in paragraphs[0].get_text()
        assert '\n' not in paragraphs[1].get_text()

    def test_timestamps_without_micro(
            self, sample_srt_timestamps_without_microseconds):
        captions = self.reader.read(sample_srt_timestamps_without_microseconds)
        first_paragraph = captions.get_captions("en-US")[0]

        assert 13000000 == first_paragraph.start
        assert 16000000 == first_paragraph.end



import pytest

from pycaption import (
    SCCReader, SCCWriter, SRTReader, SRTWriter, DFXPWriter, WebVTTWriter,
)

from tests.mixins import CaptionSetTestingMixIn

# This is quite fuzzy at the moment.
TOLERANCE_MICROSECONDS = 600 * 1000


class TestSRTtoSCCtoSRT(CaptionSetTestingMixIn):
    def _test_srt_to_scc_to_srt_conversion(self, srt_captions):
        captions_1 = SRTReader().read(srt_captions)
        scc_results = SCCWriter().write(captions_1)
        scc_captions = SCCReader().read(scc_results)
        srt_results = SRTWriter().write(scc_captions)
        captions_2 = SRTReader().read(srt_results)
        self.assert_captionset_almost_equals(
            captions_1, captions_2, TOLERANCE_MICROSECONDS
        )

    def test_srt_to_scc_to_srt_conversion(self, sample_srt_ascii):
        self._test_srt_to_scc_to_srt_conversion(sample_srt_ascii)


# The following test fails -- maybe a bug with SCCReader
#    def test_srt_to_srt_unicode_conversion(self):
#        self._test_srt_to_scc_to_srt_conversion(SAMPLE_SRT_UNICODE)


class TestSCCtoDFXP:
    def test_scc_to_dfxp(self, sample_dfxp_from_scc_output,
                         sample_scc_multiple_positioning):
        caption_set = SCCReader().read(sample_scc_multiple_positioning)
        dfxp = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        assert sample_dfxp_from_scc_output == dfxp

    def test_dfxp_is_valid_xml_when_scc_source_has_weird_italic_commands(
            self, sample_dfxp_with_properly_closing_spans_output,
            sample_scc_created_dfxp_with_wrongly_closing_spans):
        caption_set = SCCReader().read(
            sample_scc_created_dfxp_with_wrongly_closing_spans
        )

        dfxp = DFXPWriter().write(caption_set)

        assert dfxp == sample_dfxp_with_properly_closing_spans_output

    def test_dfxp_is_valid_xml_when_scc_source_has_ampersand_character(
            self, sample_dfxp_with_ampersand_character,
            sample_scc_with_ampersand_character):
        caption_set = SCCReader().read(sample_scc_with_ampersand_character)

        dfxp = DFXPWriter().write(caption_set)

        assert dfxp == sample_dfxp_with_ampersand_character


class TestSCCToWebVTT:
    def test_webvtt_newlines_are_properly_rendered(
            self, sample_webvtt_from_scc_properly_writes_newlines_output,
            scc_that_generates_webvtt_with_proper_newlines):
        caption_set = SCCReader().read(
            scc_that_generates_webvtt_with_proper_newlines)
        webvtt = WebVTTWriter().write(caption_set)

        assert webvtt == sample_webvtt_from_scc_properly_writes_newlines_output


class TestSCCEOCNewline:
    def test_eoc_on_newline_rejection(self, sample_scc_flashing_captions):
        with pytest.raises(ValueError):
            SCCReader().read(sample_scc_flashing_captions)



from tests.fixtures.dfxp import (  # noqa: F401
    sample_dfxp, sample_dfxp_with_inline_style, sample_dfxp_with_defined_style,
    sample_dfxp_with_inherited_style, sample_dfxp_without_region_and_style,
    sample_dfxp_with_positioning, sample_dfxp_with_relativized_positioning,
    sample_dfxp_empty, sample_dfxp_syntax_error,
    sample_dfxp_from_sami_with_positioning,
    sample_dfxp_long_cue, sample_dfxp_long_cue_fit_to_screen,
    sample_dfxp_from_sami_with_margins, sample_dfxp_from_sami_with_lang_margins,
    sample_dfxp_from_sami_with_span, sample_dfxp_from_sami_with_bad_span_align,
    sample_dfxp_invalid_but_supported_positioning_input,
    sample_dfxp_invalid_but_supported_positioning_output,
    sample_dfxp_multiple_regions_input, sample_dfxp_multiple_regions_output,
    sample_dfxp_to_render_with_only_default_positioning_input,
    sample_dfxp_output, sample_dfxp_style_tag_with_no_xml_id_input,
    sample_dfxp_style_tag_with_no_xml_id_output, sample_dfxp_from_scc_output,
    sample_dfxp_with_properly_closing_spans_output,
    sample_dfxp_for_legacy_writer_input, sample_dfxp_for_legacy_writer_output,
    sample_dfxp_with_templated_style, sample_dfxp_with_escaped_apostrophe,
    sample_dfxp_with_alternative_timing_formats, sample_dfxp_empty_paragraph,
    sample_dfxp_incorrect_time_format, sample_dfxp_with_frame_timing,
    sample_dfxp_empty_cue, sample_dfxp_empty_cue_output,
    sample_dfxp_invalid_positioning_value_template,
    sample_dfxp_multiple_captions_with_the_same_timing,
    sample_dfxp_with_ampersand_character, sample_dfxp_with_nested_spans,
    dfxp_style_region_align_conflict, dfxp_with_concurrent_captions,
)
from tests.fixtures.microdvd import (  # noqa: F401
    sample_microdvd, sample_microdvd_2,
    sample_microdvd_invalid_format, missing_fps_sample_microdvd,
    sample_microdvd_empty, sample_microdvd_empty_cue_output,
)
from tests.fixtures.sami import (  # noqa: F401
    sample_sami, sample_sami_with_style_tags,
    sample_sami_with_css_inline_style, sample_sami_with_css_id_style,
    sample_sami_empty, sample_sami_syntax_error,
    sample_sami_double_br, sample_sami_partial_margins,
    sample_sami_partial_margins_relativized, sample_sami_lang_margin,
    sample_sami_with_span, sample_sami_with_bad_span_align,
    sample_sami_with_bad_div_align, sample_sami_with_p_align,
    sample_sami_with_p_and_span_align, sample_sami_with_multiple_span_aligns,
    sample_sami_no_lang, sample_sami_with_lang, sample_sami_with_multi_lang,
    sample_sami_with_multiple_p, sample_sami_empty_cue_output,
    sample_sami_with_invalid_inline_style,
    sample_sami_including_hexadecimal_charref,
    sample_sami_including_decimal_charref,
    sample_sami_including_html5_entityref, sample_sami_with_unclosed_tag,
    sample_sami_with_inline_lang, sample_sami_from_dfxp_with_nested_spans,
    sample_sami_with_separate_multi_lang,
)
from tests.fixtures.scc import (  # noqa: F401
    sample_scc_created_dfxp_with_wrongly_closing_spans,
    scc_that_generates_webvtt_with_proper_newlines,
    sample_scc_produces_captions_with_start_and_end_time_the_same,
    sample_scc_pop_on, sample_scc_multiple_positioning, sample_scc_with_italics,
    sample_scc_empty, sample_scc_roll_up_ru2, sample_no_positioning_at_all_scc,
    sample_scc_no_explicit_end_to_last_caption, sample_scc_flashing_captions,
    sample_scc_eoc_first_command, sample_scc_with_extended_characters,
    sample_scc_with_ampersand_character, sample_scc_multiple_formats,
    sample_scc_repeated_tab_offset, sample_scc_tab_offset,
    sample_scc_with_unknown_commands
)
from tests.fixtures.srt import (  # noqa: F401
    sample_srt, sample_srt_ascii, sample_srt_numeric, sample_srt_empty,
    sample_srt_blank_lines, sample_srt_trailing_blanks,
    samples_srt_same_time, sample_srt_empty_cue_output,
    sample_srt_timestamps_without_microseconds,
)
from tests.fixtures.translated_scc import (  # noqa: F401
    sample_translated_scc_custom_brackets, sample_translated_scc_success,
    sample_translated_scc_commands_not_found, sample_translated_scc_no_brackets
)
from tests.fixtures.webvtt import (  # noqa: F401
    sample_webvtt, sample_webvtt_from_dfxp, sample_webvtt_from_sami,
    sample_webvtt_from_sami_with_style, sample_webvtt_from_sami_with_id_style,
    sample_webvtt_from_dfxp_with_style,
    sample_webvtt_from_dfxp_with_positioning,
    sample_webvtt_from_dfxp_with_positioning_and_style,
    sample_webvtt_from_srt, sample_webvtt_from_webvtt,
    sample_webvtt_2, sample_webvtt_empty, sample_webvtt_double_br,
    sample_webvtt_output_long_cue, webvtt_from_dfxp_with_conflicting_align,
    sample_webvtt_with_cue_settings,
    sample_webvtt_from_scc_properly_writes_newlines_output,
    sample_webvtt_last_cue_zero_start, sample_webvtt_empty_cue,
    sample_webvtt_multi_lang_en, sample_webvtt_multi_lang_de,
    sample_webvtt_empty_cue_output, sample_webvtt_timestamps
)



import pytest

from pycaption import DFXPReader, CaptionReadNoCaptions
from pycaption.exceptions import (
    CaptionReadSyntaxError, InvalidInputError, CaptionReadError,
)
from pycaption.geometry import (
    UnitEnum, HorizontalAlignmentEnum, VerticalAlignmentEnum,
)
from tests.mixins import ReaderTestingMixIn


class TestDFXPReader(ReaderTestingMixIn):
    def setup_class(self):
        self.reader = DFXPReader()

    def test_positive_answer_for_detection(self, sample_dfxp):
        super().assert_positive_answer_for_detection(sample_dfxp)

    @pytest.mark.parametrize('different_sample', [
        pytest.lazy_fixture('sample_microdvd'),
        pytest.lazy_fixture('sample_sami'),
        pytest.lazy_fixture('sample_scc_pop_on'),
        pytest.lazy_fixture('sample_srt'),
        pytest.lazy_fixture('sample_webvtt')
    ])
    def test_negative_answer_for_detection(self, different_sample):
        super().assert_negative_answer_for_detection(different_sample)

    def test_caption_length(self, sample_dfxp):
        captions = DFXPReader().read(sample_dfxp)

        assert 7 == len(captions.get_captions("en-US"))

    def test_proper_timestamps(self, sample_dfxp):
        captions = DFXPReader().read(sample_dfxp)
        paragraph = captions.get_captions("en-US")[2]

        assert 17000000 == paragraph.start
        assert 18752000 == paragraph.end

    def test_incorrect_time_format(self, sample_dfxp_incorrect_time_format):
        with pytest.raises(CaptionReadSyntaxError):
            DFXPReader().read(sample_dfxp_incorrect_time_format)

    def test_offset_time(self):
        reader = DFXPReader()

        assert 1 == reader._translate_time("0.001ms")
        assert 2000 == reader._translate_time("2ms")
        assert 1000000 == reader._translate_time("1s")
        assert 1234567 == reader._translate_time("1.234567s")
        assert 180000000 == reader._translate_time("3m")
        assert 14400000000 == reader._translate_time("4h")
        # Tick values are not supported
        with pytest.raises(InvalidInputError):
            reader._translate_time("2.3t")

    def test_empty_file(self, sample_dfxp_empty):
        with pytest.raises(CaptionReadNoCaptions):
            DFXPReader().read(sample_dfxp_empty)

    def test_invalid_markup_is_properly_handled(self, sample_dfxp_syntax_error):
        captions = DFXPReader().read(sample_dfxp_syntax_error)

        assert 2 == len(captions.get_captions("en"))

    def test_caption_error_for_invalid_positioning_values(
            self, sample_dfxp_invalid_positioning_value_template):
        invalid_value_dfxp = (
            sample_dfxp_invalid_positioning_value_template.
            format(origin="px 5px")
        )
        with pytest.raises(CaptionReadSyntaxError):
            DFXPReader().read(invalid_value_dfxp)

    def test_caption_error_for_invalid_or_unsupported_positioning_units(
            self, sample_dfxp_invalid_positioning_value_template):
        invalid_dfxp = sample_dfxp_invalid_positioning_value_template.format(
            origin="6foo 7bar"
        )
        with pytest.raises(CaptionReadSyntaxError):
            DFXPReader().read(invalid_dfxp)

    def test_individual_timings_of_captions_with_matching_timespec_are_kept(
        self, sample_dfxp_multiple_captions_with_the_same_timing
    ):
        captionset = DFXPReader().read(
            sample_dfxp_multiple_captions_with_the_same_timing
        )
        expected_timings = [(9209000, 12312000)] * 3
        actual_timings = [(c_.start, c_.end) for c_ in
                          captionset.get_captions('en-US')]

        assert expected_timings == actual_timings

    def test_individual_texts_of_captions_with_matching_timespec_are_kept(
            self, sample_dfxp_multiple_captions_with_the_same_timing):
        captionset = DFXPReader().read(
            sample_dfxp_multiple_captions_with_the_same_timing
        )

        expected_texts = ['Some text here',
                          'Some text there',
                          'Caption texts are everywhere!']
        actual_texts = [c_.nodes[0].content for c_ in
                        captionset.get_captions("en-US")]

        assert expected_texts == actual_texts

    def test_individual_layouts_of_captions_with_matching_timespec_are_kept(
        self, sample_dfxp_multiple_captions_with_the_same_timing
    ):
        captionset = DFXPReader().read(
            sample_dfxp_multiple_captions_with_the_same_timing
        )
        expected_layouts = [
            (((10, UnitEnum.PERCENT), (10, UnitEnum.PERCENT)), None, None,
             (HorizontalAlignmentEnum.CENTER, VerticalAlignmentEnum.BOTTOM)),
            (((40, UnitEnum.PERCENT), (40, UnitEnum.PERCENT)), None, None,
             (HorizontalAlignmentEnum.CENTER, VerticalAlignmentEnum.BOTTOM)),
            (((10, UnitEnum.PERCENT), (70, UnitEnum.PERCENT)), None, None,
             (HorizontalAlignmentEnum.CENTER, VerticalAlignmentEnum.BOTTOM))]
        actual_layouts = [c_.layout_info.serialized() for c_ in
                          captionset.get_captions('en-US')]

        assert expected_layouts == actual_layouts

    def test_properly_converts_timing(
            self, sample_dfxp_with_alternative_timing_formats):
        caption_set = DFXPReader().read(
            sample_dfxp_with_alternative_timing_formats)
        caps = caption_set.get_captions('en-US')

        assert caps[0].start == 1900000
        assert caps[0].end == 3050000
        assert caps[1].start == 4000000
        assert caps[1].end == 5200000

    def test_empty_paragraph(self, sample_dfxp_empty_paragraph):
        try:
            DFXPReader().read(sample_dfxp_empty_paragraph)
        except CaptionReadError:
            pytest.fail("Failing on empty paragraph")

    def test_properly_converts_frametiming(self, sample_dfxp_with_frame_timing):
        caption_set = DFXPReader().read(sample_dfxp_with_frame_timing)
        caps = caption_set.get_captions('en-US')

        assert caps[0].end == 12233333
        assert caps[0].start == 9666666

    def test_empty_cue(self, sample_dfxp_empty_cue):
        caption_set = DFXPReader().read(sample_dfxp_empty_cue)
        caps = caption_set.get_captions('en-US')

        assert len(caps) == 1



import pytest

from pycaption.geometry import Size, Point, Stretch, Padding, UnitEnum, Layout


class TestIsValidGeometryObject:
    def test_size_is_valid(self):
        with pytest.raises(TypeError):
            Size()

        with pytest.raises(ValueError):
            Size(None, None)

    def test_point_is_valid(self):
        with pytest.raises(TypeError):
            Point()

        with pytest.raises(ValueError):
            Point(None, None)

    def test_stretch_is_valid(self):
        with pytest.raises(TypeError):
            Stretch()

        with pytest.raises(ValueError):
            Stretch(None, None)


class TestIsRelative:
    def test_size_is_relative(self):
        size_px = Size(30, UnitEnum.PIXEL)
        size_percent = Size(30, UnitEnum.PERCENT)

        assert not size_px.is_relative()
        assert size_percent.is_relative()

    def test_point_is_relative(self):
        size_px = Size(30, UnitEnum.PIXEL)
        size_px2 = Size(30, UnitEnum.PIXEL)

        size_percent = Size(30, UnitEnum.PERCENT)
        size_percent2 = Size(30, UnitEnum.PERCENT)

        point_abs = Point(size_px, size_px2)
        point_mix = Point(size_percent, size_px)
        point_rel = Point(size_percent, size_percent2)

        assert not point_abs.is_relative()
        assert not point_mix.is_relative()
        assert point_rel.is_relative()

    def test_stretch_is_relative(self):
        size_px = Size(30, UnitEnum.PIXEL)
        size_px2 = Size(30, UnitEnum.PIXEL)

        size_percent = Size(30, UnitEnum.PERCENT)
        size_percent2 = Size(30, UnitEnum.PERCENT)

        stretch_abs = Stretch(size_px, size_px2)
        stretch_mix = Stretch(size_percent, size_px)
        stretch_rel = Stretch(size_percent, size_percent2)

        assert not stretch_abs.is_relative()
        assert not stretch_mix.is_relative()
        assert stretch_rel.is_relative()

    def test_padding_is_relative(self):
        size_px = Size(30, UnitEnum.PIXEL)
        size_px2 = Size(30, UnitEnum.PIXEL)
        size_px3 = Size(30, UnitEnum.PIXEL)
        size_px4 = Size(30, UnitEnum.PIXEL)

        size_percent = Size(30, UnitEnum.PERCENT)
        size_percent2 = Size(30, UnitEnum.PERCENT)
        size_percent3 = Size(30, UnitEnum.PERCENT)
        size_percent4 = Size(30, UnitEnum.PERCENT)

        padding_abs = Padding(size_px, size_px2, size_px3, size_px4)
        padding_mix = Padding(size_px, size_px2, size_px3, size_percent)
        padding_rel = Padding(
            size_percent, size_percent2, size_percent3, size_percent4)

        assert not padding_abs.is_relative()
        assert not padding_mix.is_relative()
        assert padding_rel.is_relative()

    def test_layout_is_relative(self):
        empty_layout = Layout()

        size_px = Size(30, UnitEnum.PIXEL)
        size_px2 = Size(30, UnitEnum.PIXEL)

        size_percent = Size(30, UnitEnum.PERCENT)
        size_percent2 = Size(30, UnitEnum.PERCENT)

        point_abs = Point(size_px, size_px2)
        point_rel = Point(size_percent, size_percent2)

        stretch_abs = Stretch(size_px, size_px2)
        stretch_rel = Stretch(size_percent, size_percent2)

        layout_abs = Layout(
            origin=point_abs,
            extent=stretch_abs,
            padding=None
        )

        layout_mix = Layout(
            origin=point_abs,
            extent=stretch_rel,
            padding=None
        )

        layout_rel = Layout(
            origin=point_rel,
            extent=stretch_rel,
            padding=None
        )

        assert empty_layout.is_relative()
        assert not layout_abs.is_relative()
        assert not layout_mix.is_relative()
        assert layout_rel.is_relative()



import re

from pycaption import (
    SAMIReader, SRTReader, WebVTTReader, WebVTTWriter, DFXPWriter,
    MicroDVDWriter,
)

from tests.mixins import (
    WebVTTTestingMixIn, DFXPTestingMixIn, MicroDVDTestingMixIn,
)


class TestSAMItoWebVTT(WebVTTTestingMixIn):
    def test_sami_to_webvtt_conversion(
            self, sample_webvtt_from_sami, sample_sami):
        caption_set = SAMIReader().read(sample_sami)
        results = WebVTTWriter(
            video_width=640, video_height=360).write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_sami, results)

    def test_sami_with_style_tags_to_webvtt_conversion(
            self, sample_webvtt_from_sami_with_style,
            sample_sami_with_style_tags):
        caption_set = SAMIReader().read(sample_sami_with_style_tags)
        results = WebVTTWriter(
            video_width=640, video_height=360).write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_sami_with_style, results)

    def test_sami_with_css_inline_style_to_webvtt_conversion(
            self, sample_webvtt_from_sami_with_style,
            sample_sami_with_css_inline_style):
        caption_set = SAMIReader().read(sample_sami_with_css_inline_style)
        results = WebVTTWriter(
            video_width=640, video_height=360).write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_sami_with_style, results)

    def test_sami_with_css_id_style_to_webvtt_conversion(
            self, sample_webvtt_from_sami_with_id_style,
            sample_sami_with_css_id_style):
        caption_set = SAMIReader().read(sample_sami_with_css_id_style)
        results = WebVTTWriter(
            video_width=640, video_height=360).write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_sami_with_id_style,
                                  results)


class TestSRTtoWebVTT(WebVTTTestingMixIn):
    def test_srt_to_webvtt_conversion(self, sample_webvtt_from_srt, sample_srt):
        caption_set = SRTReader().read(sample_srt)
        results = WebVTTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_srt, results)


class TestWebVTTtoWebVTT(WebVTTTestingMixIn):
    def test_webvtt_to_webvtt_conversion(self, sample_webvtt_from_webvtt,
                                         sample_webvtt):
        caption_set = WebVTTReader().read(sample_webvtt)
        results = WebVTTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_webvtt, results)

    def test_cue_settings_are_kept(self, sample_webvtt_with_cue_settings):
        caption_set = WebVTTReader().read(sample_webvtt_with_cue_settings)

        webvtt = WebVTTWriter().write(caption_set)

        assert sample_webvtt_with_cue_settings == webvtt

    def test_positioning_is_kept(self,
                                 sample_webvtt_from_dfxp_with_positioning):
        caption_set = WebVTTReader().read(
            sample_webvtt_from_dfxp_with_positioning)
        results = WebVTTWriter().write(caption_set)

        assert sample_webvtt_from_dfxp_with_positioning == results

    def test_output_timestamps(self, sample_webvtt_timestamps):
        expected_timestamp_line_pattern = re.compile(
            r'^(\d{2,}):(\d{2})(:\d{2})?\.(\d{3}) '
            r'--> (\d{2,}):(\d{2})(:\d{2})?\.(\d{3})')

        caption_set = WebVTTReader().read(sample_webvtt_timestamps)
        results = WebVTTWriter().write(caption_set).splitlines()

        assert re.match(expected_timestamp_line_pattern, results[2])
        assert re.match(expected_timestamp_line_pattern, results[5])

#     # TODO: Write a test that includes a WebVTT file with style tags
#     # That will fail because the styles used in the cues are not tracked.


class TestWebVTTtoDFXP(DFXPTestingMixIn):
    def test_webvtt_to_dfxp_conversion(self, sample_dfxp, sample_webvtt):
        caption_set = WebVTTReader().read(sample_webvtt)
        results = DFXPWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_dfxp_equals(
            sample_dfxp, results, ignore_styling=True, ignore_spans=True
        )


class TestWebVTTtoMicroDVD(MicroDVDTestingMixIn):
    def test_webvtt_to_microdvd_conversion(self, sample_microdvd,
                                           sample_webvtt):
        caption_set = WebVTTReader().read(sample_webvtt)
        results = MicroDVDWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_microdvd_equals(sample_microdvd, results)



from pycaption import DFXPReader
from pycaption.base import merge_concurrent_captions


class TestFunctions:
    def test_merge_concurrent_captions(self, dfxp_with_concurrent_captions):
        initial_caption_set = DFXPReader().read(dfxp_with_concurrent_captions)
        initial_captions = initial_caption_set.get_captions('en-US')
        caption_set = merge_concurrent_captions(initial_caption_set)
        captions = caption_set.get_captions('en-US')

        assert len(initial_captions) == 5
        assert len(captions) == 3



import pytest

from pycaption import (
    WebVTTReader, WebVTTWriter, SAMIReader, DFXPReader,
    CaptionReadNoCaptions, CaptionReadError, CaptionReadSyntaxError,
)
from tests.mixins import ReaderTestingMixIn


class TestWebVTTReader(ReaderTestingMixIn):
    def setup_method(self):
        self.reader = WebVTTReader()

    def test_positive_answer_for_detection(self, sample_webvtt):
        super().assert_positive_answer_for_detection(sample_webvtt)

    @pytest.mark.parametrize('different_sample', [
        pytest.lazy_fixture('sample_dfxp'),
        pytest.lazy_fixture('sample_microdvd'),
        pytest.lazy_fixture('sample_sami'),
        pytest.lazy_fixture('sample_scc_pop_on'),
        pytest.lazy_fixture('sample_srt')
    ])
    def test_negative_answer_for_detection(self, different_sample):
        super().assert_negative_answer_for_detection(different_sample)

    def test_caption_length(self, sample_webvtt_2):
        captions = self.reader.read(sample_webvtt_2)

        assert len(captions.get_captions('en-US')) == 7

    def test_read_supports_multiple_languages(self, sample_webvtt):
        captions = self.reader.read(sample_webvtt, lang='es')

        assert captions.get_captions('es') is not None

    def test_proper_timestamps(self, sample_webvtt):
        captions = self.reader.read(sample_webvtt)
        cue = captions.get_captions('en-US')[2]

        assert cue.start == 17000000
        assert cue.end == 18752000

    def test_forward_time_shift(self, sample_webvtt):
        captions = WebVTTReader(time_shift_milliseconds=15).read(sample_webvtt)
        cue = captions.get_captions('en-US')[2]

        assert cue.start == 17015000
        assert cue.end == 18767000

    def test_backward_time_shift(self, sample_webvtt):
        captions = WebVTTReader(time_shift_milliseconds=-15).read(sample_webvtt)
        cue = captions.get_captions('en-US')[2]

        assert cue.start == 16985000
        assert cue.end == 18737000

    def test_webvtt_cue_components_removed_from_text(self):
        result = self.reader._remove_styles(
            "<c vIntro><b>Wikipedia</b> is a great adventure. <i>It may have "
            "its shortcomings</i>, but it is<u> the largest</u> collective "
            "knowledge construction endevour</c> <ruby>base text <rt>"
            "annotation</rt></ruby> <v Audry><b>Yes</b>, indeed!"
        )
        expected = (
            "Wikipedia is a great adventure. It may have "
            "its shortcomings, but it is the largest collective "
            "knowledge construction endevour base text annotation"
            " Audry: Yes, indeed!"
        )
        assert result == expected

    def test_empty_file(self, sample_webvtt_empty):
        with pytest.raises(CaptionReadNoCaptions):
            WebVTTReader().read(sample_webvtt_empty)

    def test_not_ignoring_timing_errors(self):
        # todo: same assert w/ different arguments -> this can be parametrized;
        with pytest.raises(CaptionReadError):
            WebVTTReader(ignore_timing_errors=False).read(
                "\n" "00:00:20.000 --> 00:00:10.000\n" "foo bar baz")

        with pytest.raises(CaptionReadError):
            WebVTTReader(ignore_timing_errors=False).read(
                "00:00:20.000 --> 00:00:10.000\n"
                "Start time is greater than end time.\n"
            )

        with pytest.raises(CaptionReadError):
            WebVTTReader(ignore_timing_errors=False).read(
                "00:00:20.000 --> 00:00:30.000\n"
                "Start times should be consecutive.\n"
                "\n"
                "00:00:10.000 --> 00:00:20.000\n"
                "This cue starts before the previous one.\n"
            )

    def test_ignoring_timing_errors(self):
        # Even if timing errors are ignored, this has to raise an exception
        with pytest.raises(CaptionReadSyntaxError):
            WebVTTReader().read(
                "\nNOTE invalid cue stamp\n00:00:20.000 --> \nfoo bar baz\n")

        # And this too
        with pytest.raises(CaptionReadSyntaxError):
            WebVTTReader().read("\n00:00:20,000 --> 00:00:22,000\n"
                                "Note the comma instead of point.\n")

        # todo: at this point it can be split into 2 separate tests
        try:
            WebVTTReader().read(
                "\n"
                "00:00:20.000 --> 00:00:10.000\n"
                "Start time is greater than end time.\n"
            )
        except CaptionReadError:
            pytest.fail("Shouldn't raise CaptionReadError")

        try:
            WebVTTReader().read(
                "\n"
                "00:00:20.000 --> 00:00:30.000\n"
                "Start times should be consecutive.\n"
                "\n"
                "00:00:10.000 --> 00:00:20.000\n"
                "This cue starts before the previous one.\n"
            )
        except CaptionReadError:
            pytest.fail("Shouldn't raise CaptionReadError")

    def test_invalid_files(self):
        with pytest.raises(CaptionReadError):
            WebVTTReader(ignore_timing_errors=False).read(
                "00:00:20.000 --> 00:00:10.000\n"
                "Start time is greater than end time.")

        with pytest.raises(CaptionReadError):
            WebVTTReader(ignore_timing_errors=False).read(
                "00:00:20.000 --> 00:00:30.000\n"
                "Start times should be consecutive.\n"
                "\n"
                "00:00:10.000 --> 00:00:20.000\n"
                "This cue starts before the previous one.\n"
            )

    def test_zero_start(self, sample_webvtt_last_cue_zero_start):
        captions = self.reader.read(sample_webvtt_last_cue_zero_start)
        cue = captions.get_captions('en-US')[0]

        assert cue.start == 0

    def test_webvtt_empty_cue(self, sample_webvtt_empty_cue):
        assert 1 == len(self.reader.read(
            sample_webvtt_empty_cue).get_captions('en-US'))


class TestWebVTTWriter:
    def setup_method(self):
        self.writer = WebVTTWriter()

    def test_double_br(self, sample_webvtt_double_br, sample_sami_double_br):
        caption_set = SAMIReader().read(sample_sami_double_br)
        results = WebVTTWriter().write(caption_set)

        assert sample_webvtt_double_br == results

    def test_break_node_positioning_is_ignored(
            self, webvtt_from_dfxp_with_conflicting_align,
            dfxp_style_region_align_conflict):
        caption_set = DFXPReader().read(dfxp_style_region_align_conflict)
        results = WebVTTWriter().write(caption_set)

        assert webvtt_from_dfxp_with_conflicting_align == results

    def test_lang_option(self, sample_webvtt_multi_lang_en,
                         sample_webvtt_multi_lang_de,
                         sample_sami_with_multi_lang):
        caption_set = SAMIReader().read(sample_sami_with_multi_lang)
        results = WebVTTWriter().write(caption_set, 'de-DE')

        assert sample_webvtt_multi_lang_de == results
        results = WebVTTWriter().write(caption_set, 'en-US')
        assert sample_webvtt_multi_lang_en == results



from pycaption.scc.constants import CHARACTERS, SPECIAL_CHARS, EXTENDED_CHARS

ALL_CHARACTERS = {**CHARACTERS, **SPECIAL_CHARS, **EXTENDED_CHARS}
COMMAND_LABELS = {
    "9420": "Resume Caption Loading",
    "9429": "Resume Direct Captioning",
    "9425": "Roll-Up Captions--2 Rows",
    "9426": "Roll-Up Captions--3 Rows",
    "94a7": "Roll-Up Captions--4 Rows",
    "942a": "Text Restart",
    "94ab": "Resume Text Display",
    "942c": "Erase Displayed Memory",
    "94ae": "Erase Non-displayed Memory",
    "942f": "End Of Caption",
    "9140": "row 01, column 00, with plain white text.",
    "91c1": "row 01, column 00, with white underlined text.",
    "91c2": "row 01, column 00, with plain green text.",
    "9143": "row 01, column 00, with green underlined text.",
    "91c4": "row 01, column 00, with plain blue text.",
    "9145": "row 01, column 00, with blue underlined text.",
    "9146": "row 01, column 00, with plain cyan text.",
    "91c7": "row 01, column 00, with cyan underlined text.",
    "91c8": "row 01, column 00, with plain red text.",
    "9149": "row 01, column 00, with red underlined text.",
    "914a": "row 01, column 00, with plain yellow text.",
    "91cb": "row 01, column 00, with yellow underlined text.",
    "914c": "row 01, column 00, with plain magenta text.",
    "91cd": "row 01, column 00, with magenta underlined text.",
    "91ce": "row 01, column 00, with white italicized text.",
    "914f": "row 01, column 00, with white underlined italicized text.",
    "91d0": "row 01, column 00, with plain white text.",
    "9151": "row 01, column 00, with white underlined text.",
    "9152": "row 01, column 04, with plain white text.",
    "91d3": "row 01, column 04, with white underlined text.",
    "9154": "row 01, column 08, with plain white text.",
    "91d5": "row 01, column 08, with white underlined text.",
    "91d6": "row 01, column 12, with plain white text.",
    "9157": "row 01, column 12, with white underlined text.",
    "9158": "row 01, column 16, with plain white text.",
    "91d9": "row 01, column 16, with white underlined text.",
    "91da": "row 01, column 20, with plain white text.",
    "915b": "row 01, column 20, with white underlined text.",
    "91dc": "row 01, column 24, with plain white text.",
    "915d": "row 01, column 24, with white underlined text.",
    "915e": "row 01, column 28, with plain white text.",
    "91df": "row 01, column 28, with white underlined text.",
    "91e0": "row 02, column 00, with plain white text.",
    "9161": "row 02, column 00, with white underlined text.",
    "9162": "row 02, column 00, with plain green text.",
    "91e3": "row 02, column 00, with green underlined text.",
    "9164": "row 02, column 00, with plain blue text.",
    "91e5": "row 02, column 00, with blue underlined text.",
    "91e6": "row 02, column 00, with plain cyan text.",
    "9167": "row 02, column 00, with cyan underlined text.",
    "9168": "row 02, column 00, with plain red text.",
    "91e9": "row 02, column 00, with red underlined text.",
    "91ea": "row 02, column 00, with plain yellow text.",
    "916b": "row 02, column 00, with yellow underlined text.",
    "91ec": "row 02, column 00, with plain magenta text.",
    "916d": "row 02, column 00, with magenta underlined text.",
    "916e": "row 02, column 00, with white italicized text.",
    "91ef": "row 02, column 00, with white underlined italicized text.",
    "9170": "row 02, column 00, with plain white text.",
    "91f1": "row 02, column 00, with white underlined text.",
    "91f2": "row 02, column 04, with plain white text.",
    "9173": "row 02, column 04, with white underlined text.",
    "91f4": "row 02, column 08, with plain white text.",
    "9175": "row 02, column 08, with white underlined text.",
    "9176": "row 02, column 12, with plain white text.",
    "91f7": "row 02, column 12, with white underlined text.",
    "91f8": "row 02, column 16, with plain white text.",
    "9179": "row 02, column 16, with white underlined text.",
    "917a": "row 02, column 20, with plain white text.",
    "91fb": "row 02, column 20, with white underlined text.",
    "91fc": "row 02, column 24, with plain white text.",
    "91fd": "row 02, column 24, with white underlined text.",
    "91fe": "row 02, column 28, with plain white text.",
    "917f": "row 02, column 28, with white underlined text.",
    "9240": "row 03, column 00, with plain white text.",
    "92c1": "row 03, column 00, with white underlined text.",
    "92c2": "row 03, column 00, with plain green text.",
    "9243": "row 03, column 00, with green underlined text.",
    "92c4": "row 03, column 00, with plain blue text.",
    "9245": "row 03, column 00, with blue underlined text.",
    "9246": "row 03, column 00, with plain cyan text.",
    "92c7": "row 03, column 00, with cyan underlined text.",
    "92c8": "row 03, column 00, with plain red text.",
    "9249": "row 03, column 00, with red underlined text.",
    "924a": "row 03, column 00, with plain yellow text.",
    "92cb": "row 03, column 00, with yellow underlined text.",
    "924c": "row 03, column 00, with plain magenta text.",
    "92cd": "row 03, column 00, with magenta underlined text.",
    "92ce": "row 03, column 00, with white italicized text.",
    "924f": "row 03, column 00, with white underlined italicized text.",
    "92d0": "row 03, column 00, with plain white text.",
    "9251": "row 03, column 00, with white underlined text.",
    "9252": "row 03, column 04, with plain white text.",
    "92d3": "row 03, column 04, with white underlined text.",
    "9254": "row 03, column 08, with plain white text.",
    "92d5": "row 03, column 08, with white underlined text.",
    "92d6": "row 03, column 12, with plain white text.",
    "9257": "row 03, column 12, with white underlined text.",
    "9258": "row 03, column 16, with plain white text.",
    "92d9": "row 03, column 16, with white underlined text.",
    "92da": "row 03, column 20, with plain white text.",
    "925b": "row 03, column 20, with white underlined text.",
    "92dc": "row 03, column 24, with plain white text.",
    "925d": "row 03, column 24, with white underlined text.",
    "925e": "row 03, column 28, with plain white text.",
    "92df": "row 03, column 28, with white underlined text.",
    "92e0": "row 04, column 00, with plain white text.",
    "9261": "row 04, column 00, with white underlined text.",
    "9262": "row 04, column 00, with plain green text.",
    "92e3": "row 04, column 00, with green underlined text.",
    "9264": "row 04, column 00, with plain blue text.",
    "92e5": "row 04, column 00, with blue underlined text.",
    "92e6": "row 04, column 00, with plain cyan text.",
    "9267": "row 04, column 00, with cyan underlined text.",
    "9268": "row 04, column 00, with plain red text.",
    "92e9": "row 04, column 00, with red underlined text.",
    "92ea": "row 04, column 00, with plain yellow text.",
    "926b": "row 04, column 00, with yellow underlined text.",
    "92ec": "row 04, column 00, with plain magenta text.",
    "926d": "row 04, column 00, with magenta underlined text.",
    "926e": "row 04, column 00, with white italicized text.",
    "92ef": "row 04, column 00, with white underlined italicized text.",
    "9270": "row 04, column 00, with plain white text.",
    "92f1": "row 04, column 00, with white underlined text.",
    "92f2": "row 04, column 04, with plain white text.",
    "9273": "row 04, column 04, with white underlined text.",
    "92f4": "row 04, column 08, with plain white text.",
    "9275": "row 04, column 08, with white underlined text.",
    "9276": "row 04, column 12, with plain white text.",
    "92f7": "row 04, column 12, with white underlined text.",
    "92f8": "row 04, column 16, with plain white text.",
    "9279": "row 04, column 16, with white underlined text.",
    "927a": "row 04, column 20, with plain white text.",
    "92fb": "row 04, column 20, with white underlined text.",
    "92fc": "row 04, column 24, with plain white text.",
    "92fd": "row 04, column 24, with white underlined text.",
    "92fe": "row 04, column 28, with plain white text.",
    "927f": "row 04, column 28, with white underlined text.",
    "1540": "row 05, column 00, with plain white text.",
    "15c1": "row 05, column 00, with white underlined text.",
    "15c2": "row 05, column 00, with plain green text.",
    "1543": "row 05, column 00, with green underlined text.",
    "15c4": "row 05, column 00, with plain blue text.",
    "1545": "row 05, column 00, with blue underlined text.",
    "1546": "row 05, column 00, with plain cyan text.",
    "15c7": "row 05, column 00, with cyan underlined text.",
    "15c8": "row 05, column 00, with plain red text.",
    "1549": "row 05, column 00, with red underlined text.",
    "154a": "row 05, column 00, with plain yellow text.",
    "15cb": "row 05, column 00, with yellow underlined text.",
    "154c": "row 05, column 00, with plain magenta text.",
    "15cd": "row 05, column 00, with magenta underlined text.",
    "15ce": "row 05, column 00, with white italicized text.",
    "154f": "row 05, column 00, with white underlined italicized text.",
    "15d0": "row 05, column 00, with plain white text.",
    "1551": "row 05, column 00, with white underlined text.",
    "1552": "row 05, column 04, with plain white text.",
    "15d3": "row 05, column 04, with white underlined text.",
    "1554": "row 05, column 08, with plain white text.",
    "15d5": "row 05, column 08, with white underlined text.",
    "15d6": "row 05, column 12, with plain white text.",
    "1557": "row 05, column 12, with white underlined text.",
    "1558": "row 05, column 16, with plain white text.",
    "15d9": "row 05, column 16, with white underlined text.",
    "15da": "row 05, column 20, with plain white text.",
    "155b": "row 05, column 20, with white underlined text.",
    "15dc": "row 05, column 24, with plain white text.",
    "155d": "row 05, column 24, with white underlined text.",
    "155e": "row 05, column 28, with plain white text.",
    "15df": "row 05, column 28, with white underlined text.",
    "15e0": "row 06, column 00, with plain white text.",
    "1561": "row 06, column 00, with white underlined text.",
    "15462": "row 06, column 00, with plain green text.",
    "15e3": "row 06, column 00, with green underlined text.",
    "1564": "row 06, column 00, with plain blue text.",
    "15e5": "row 06, column 00, with blue underlined text.",
    "15e6": "row 06, column 00, with plain cyan text.",
    "1567": "row 06, column 00, with cyan underlined text.",
    "1568": "row 06, column 00, with plain red text.",
    "15e9": "row 06, column 00, with red underlined text.",
    "15ea": "row 06, column 00, with plain yellow text.",
    "156b": "row 06, column 00, with yellow underlined text.",
    "15ec": "row 06, column 00, with plain magenta text.",
    "156d": "row 06, column 00, with magenta underlined text.",
    "156e": "row 06, column 00, with white italicized text.",
    "15ef": "row 06, column 00, with white underlined italicized text.",
    "1570": "row 06, column 00, with plain white text.",
    "15f1": "row 06, column 00, with white underlined text.",
    "15f2": "row 06, column 04, with plain white text.",
    "1573": "row 06, column 04, with white underlined text.",
    "15f4": "row 06, column 08, with plain white text.",
    "1575": "row 06, column 08, with white underlined text.",
    "1576": "row 06, column 12, with plain white text.",
    "15f7": "row 06, column 12, with white underlined text.",
    "15f8": "row 06, column 16, with plain white text.",
    "1579": "row 06, column 16, with white underlined text.",
    "157a": "row 06, column 20, with plain white text.",
    "15fb": "row 06, column 20, with white underlined text.",
    "15fc": "row 06, column 24, with plain white text.",
    "15fd": "row 06, column 24, with white underlined text.",
    "15fe": "row 06, column 28, with plain white text.",
    "157f": "row 06, column 28, with white underlined text.",
    "1640": "row 07, column 00, with plain white text.",
    "16c1": "row 07, column 00, with white underlined text.",
    "16c2": "row 07, column 00, with plain green text.",
    "1643": "row 07, column 00, with green underlined text.",
    "16c4": "row 07, column 00, with plain blue text.",
    "1645": "row 07, column 00, with blue underlined text.",
    "1646": "row 07, column 00, with plain cyan text.",
    "16c7": "row 07, column 00, with cyan underlined text.",
    "16c8": "row 07, column 00, with plain red text.",
    "1649": "row 07, column 00, with red underlined text.",
    "164a": "row 07, column 00, with plain yellow text.",
    "16cb": "row 07, column 00, with yellow underlined text.",
    "164c": "row 07, column 00, with plain magenta text.",
    "16cd": "row 07, column 00, with magenta underlined text.",
    "16ce": "row 07, column 00, with white italicized text.",
    "164f": "row 07, column 00, with white underlined italicized text.",
    "16d0": "row 07, column 00, with plain white text.",
    "1651": "row 07, column 00, with white underlined text.",
    "1652": "row 07, column 04, with plain white text.",
    "16d3": "row 07, column 04, with white underlined text.",
    "1654": "row 07, column 08, with plain white text.",
    "16d5": "row 07, column 08, with white underlined text.",
    "16d6": "row 07, column 12, with plain white text.",
    "1657": "row 07, column 12, with white underlined text.",
    "1658": "row 07, column 16, with plain white text.",
    "16d9": "row 07, column 16, with white underlined text.",
    "16da": "row 07, column 20, with plain white text.",
    "165b": "row 07, column 20, with white underlined text.",
    "16dc": "row 07, column 24, with plain white text.",
    "165d": "row 07, column 24, with white underlined text.",
    "165e": "row 07, column 28, with plain white text.",
    "16df": "row 07, column 28, with white underlined text.",
    "16e0": "row 08, column 00, with plain white text.",
    "1661": "row 08, column 00, with white underlined text.",
    "16462": "row 08, column 00, with plain green text.",
    "16e3": "row 08, column 00, with green underlined text.",
    "1664": "row 08, column 00, with plain blue text.",
    "16e5": "row 08, column 00, with blue underlined text.",
    "16e6": "row 08, column 00, with plain cyan text.",
    "1667": "row 08, column 00, with cyan underlined text.",
    "1668": "row 08, column 00, with plain red text.",
    "16e9": "row 08, column 00, with red underlined text.",
    "16ea": "row 08, column 00, with plain yellow text.",
    "166b": "row 08, column 00, with yellow underlined text.",
    "16ec": "row 08, column 00, with plain magenta text.",
    "166d": "row 08, column 00, with magenta underlined text.",
    "166e": "row 08, column 00, with white italicized text.",
    "16ef": "row 08, column 00, with white underlined italicized text.",
    "1670": "row 08, column 00, with plain white text.",
    "16f1": "row 08, column 00, with white underlined text.",
    "16f2": "row 08, column 04, with plain white text.",
    "1673": "row 08, column 04, with white underlined text.",
    "16f4": "row 08, column 08, with plain white text.",
    "1675": "row 08, column 08, with white underlined text.",
    "1676": "row 08, column 12, with plain white text.",
    "16f7": "row 08, column 12, with white underlined text.",
    "16f8": "row 08, column 16, with plain white text.",
    "1679": "row 08, column 16, with white underlined text.",
    "167a": "row 08, column 20, with plain white text.",
    "16fb": "row 08, column 20, with white underlined text.",
    "16fc": "row 08, column 24, with plain white text.",
    "16fd": "row 08, column 24, with white underlined text.",
    "16fe": "row 08, column 28, with plain white text.",
    "167f": "row 08, column 28, with white underlined text.",
    "9740": "row 09, column 00, with plain white text.",
    "97c1": "row 09, column 00, with white underlined text.",
    "97c2": "row 09, column 00, with plain green text.",
    "9743": "row 09, column 00, with green underlined text.",
    "97c4": "row 09, column 00, with plain blue text.",
    "9745": "row 09, column 00, with blue underlined text.",
    "9746": "row 09, column 00, with plain cyan text.",
    "97c7": "row 09, column 00, with cyan underlined text.",
    "97c8": "row 09, column 00, with plain red text.",
    "9749": "row 09, column 00, with red underlined text.",
    "974a": "row 09, column 00, with plain yellow text.",
    "97cb": "row 09, column 00, with yellow underlined text.",
    "974c": "row 09, column 00, with plain magenta text.",
    "97cd": "row 09, column 00, with magenta underlined text.",
    "97ce": "row 09, column 00, with white italicized text.",
    "974f": "row 09, column 00, with white underlined italicized text.",
    "97d0": "row 09, column 00, with plain white text.",
    "9751": "row 09, column 00, with white underlined text.",
    "9752": "row 09, column 04, with plain white text.",
    "97d3": "row 09, column 04, with white underlined text.",
    "9754": "row 09, column 08, with plain white text.",
    "97d5": "row 09, column 08, with white underlined text.",
    "97d6": "row 09, column 12, with plain white text.",
    "9757": "row 09, column 12, with white underlined text.",
    "9758": "row 09, column 16, with plain white text.",
    "97d9": "row 09, column 16, with white underlined text.",
    "97da": "row 09, column 20, with plain white text.",
    "975b": "row 09, column 20, with white underlined text.",
    "97dc": "row 09, column 24, with plain white text.",
    "975d": "row 09, column 24, with white underlined text.",
    "975e": "row 09, column 28, with plain white text.",
    "97df": "row 09, column 28, with white underlined text.",
    "97e0": "row 10, column 00, with plain white text.",
    "9761": "row 10, column 00, with white underlined text.",
    "9762": "row 10, column 00, with plain green text.",
    "97e3": "row 10, column 00, with green underlined text.",
    "9764": "row 10, column 00, with plain blue text.",
    "97e5": "row 10, column 00, with blue underlined text.",
    "97e6": "row 10, column 00, with plain cyan text.",
    "9767": "row 10, column 00, with cyan underlined text.",
    "9768": "row 10, column 00, with plain red text.",
    "97e9": "row 10, column 00, with red underlined text.",
    "97ea": "row 10, column 00, with plain yellow text.",
    "976b": "row 10, column 00, with yellow underlined text.",
    "97ec": "row 10, column 00, with plain magenta text.",
    "976d": "row 10, column 00, with magenta underlined text.",
    "976e": "row 10, column 00, with white italicized text.",
    "97ef": "row 10, column 00, with white underlined italicized text.",
    "9770": "row 10, column 00, with plain white text.",
    "97f1": "row 10, column 00, with white underlined text.",
    "97f2": "row 10, column 04, with plain white text.",
    "9773": "row 10, column 04, with white underlined text.",
    "97f4": "row 10, column 08, with plain white text.",
    "9775": "row 10, column 08, with white underlined text.",
    "9776": "row 10, column 12, with plain white text.",
    "97f7": "row 10, column 12, with white underlined text.",
    "97f8": "row 10, column 16, with plain white text.",
    "9779": "row 10, column 16, with white underlined text.",
    "977a": "row 10, column 20, with plain white text.",
    "97fb": "row 10, column 20, with white underlined text.",
    "97fc": "row 10, column 24, with plain white text.",
    "97fd": "row 10, column 24, with white underlined text.",
    "97fe": "row 10, column 28, with plain white text.",
    "977f": "row 10, column 28, with white underlined text.",
    "1040": "row 11, column 00, with plain white text.",
    "10c1": "row 11, column 00, with white underlined text.",
    "10c2": "row 11, column 00, with plain green text.",
    "1043": "row 11, column 00, with green underlined text.",
    "10c4": "row 11, column 00, with plain blue text.",
    "1045": "row 11, column 00, with blue underlined text.",
    "1046": "row 11, column 00, with plain cyan text.",
    "10c7": "row 11, column 00, with cyan underlined text.",
    "10c8": "row 11, column 00, with plain red text.",
    "1049": "row 11, column 00, with red underlined text.",
    "104a": "row 11, column 00, with plain yellow text.",
    "10cb": "row 11, column 00, with yellow underlined text.",
    "104c": "row 11, column 00, with plain magenta text.",
    "10cd": "row 11, column 00, with magenta underlined text.",
    "10ce": "row 11, column 00, with white italicized text.",
    "104f": "row 11, column 00, with white underlined italicized text.",
    "10d0": "row 11, column 00, with plain white text.",
    "1051": "row 11, column 00, with white underlined text.",
    "1052": "row 11, column 04, with plain white text.",
    "10d3": "row 11, column 04, with white underlined text.",
    "1054": "row 11, column 08, with plain white text.",
    "10d5": "row 11, column 08, with white underlined text.",
    "10d6": "row 11, column 12, with plain white text.",
    "1057": "row 11, column 12, with white underlined text.",
    "1058": "row 11, column 16, with plain white text.",
    "10d9": "row 11, column 16, with white underlined text.",
    "10da": "row 11, column 20, with plain white text.",
    "105b": "row 11, column 20, with white underlined text.",
    "10dc": "row 11, column 24, with plain white text.",
    "105d": "row 11, column 24, with white underlined text.",
    "105e": "row 11, column 28, with plain white text.",
    "10df": "row 11, column 28, with white underlined text.",
    "1340": "row 12, column 00, with plain white text.",
    "13c1": "row 12, column 00, with white underlined text.",
    "13c2": "row 12, column 00, with plain green text.",
    "1343": "row 12, column 00, with green underlined text.",
    "13c4": "row 12, column 00, with plain blue text.",
    "1345": "row 12, column 00, with blue underlined text.",
    "1346": "row 12, column 00, with plain cyan text.",
    "13c7": "row 12, column 00, with cyan underlined text.",
    "13c8": "row 12, column 00, with plain red text.",
    "1349": "row 12, column 00, with red underlined text.",
    "134a": "row 12, column 00, with plain yellow text.",
    "13cb": "row 12, column 00, with yellow underlined text.",
    "134c": "row 12, column 00, with plain magenta text.",
    "13cd": "row 12, column 00, with magenta underlined text.",
    "13ce": "row 12, column 00, with white italicized text.",
    "134f": "row 12, column 00, with white underlined italicized text.",
    "13d0": "row 12, column 00, with plain white text.",
    "1351": "row 12, column 00, with white underlined text.",
    "1352": "row 12, column 04, with plain white text.",
    "13d3": "row 12, column 04, with white underlined text.",
    "1354": "row 12, column 08, with plain white text.",
    "13d5": "row 12, column 08, with white underlined text.",
    "13d6": "row 12, column 12, with plain white text.",
    "1357": "row 12, column 12, with white underlined text.",
    "1358": "row 12, column 16, with plain white text.",
    "13d9": "row 12, column 16, with white underlined text.",
    "13da": "row 12, column 20, with plain white text.",
    "135b": "row 12, column 20, with white underlined text.",
    "13dc": "row 12, column 24, with plain white text.",
    "135d": "row 12, column 24, with white underlined text.",
    "135e": "row 12, column 28, with plain white text.",
    "13df": "row 12, column 28, with white underlined text.",
    "13e0": "row 13, column 00, with plain white text.",
    "1361": "row 13, column 00, with white underlined text.",
    "13462": "row 13, column 00, with plain green text.",
    "13e3": "row 13, column 00, with green underlined text.",
    "1364": "row 13, column 00, with plain blue text.",
    "13e5": "row 13, column 00, with blue underlined text.",
    "13e6": "row 13, column 00, with plain cyan text.",
    "1367": "row 13, column 00, with cyan underlined text.",
    "1368": "row 13, column 00, with plain red text.",
    "13e9": "row 13, column 00, with red underlined text.",
    "13ea": "row 13, column 00, with plain yellow text.",
    "136b": "row 13, column 00, with yellow underlined text.",
    "13ec": "row 13, column 00, with plain magenta text.",
    "136d": "row 13, column 00, with magenta underlined text.",
    "136e": "row 13, column 00, with white italicized text.",
    "13ef": "row 13, column 00, with white underlined italicized text.",
    "1370": "row 13, column 00, with plain white text.",
    "13f1": "row 13, column 00, with white underlined text.",
    "13f2": "row 13, column 04, with plain white text.",
    "1373": "row 13, column 04, with white underlined text.",
    "13f4": "row 13, column 08, with plain white text.",
    "1375": "row 13, column 08, with white underlined text.",
    "1376": "row 13, column 12, with plain white text.",
    "13f7": "row 13, column 12, with white underlined text.",
    "13f8": "row 13, column 16, with plain white text.",
    "1379": "row 13, column 16, with white underlined text.",
    "137a": "row 13, column 20, with plain white text.",
    "13fb": "row 13, column 20, with white underlined text.",
    "13fc": "row 13, column 24, with plain white text.",
    "13fd": "row 13, column 24, with white underlined text.",
    "13fe": "row 13, column 28, with plain white text.",
    "137f": "row 13, column 28, with white underlined text.",
    "9440": "row 14, column 00, with plain white text.",
    "94c1": "row 14, column 00, with white underlined text.",
    "94c2": "row 14, column 00, with plain green text.",
    "9443": "row 14, column 00, with green underlined text.",
    "94c4": "row 14, column 00, with plain blue text.",
    "9445": "row 14, column 00, with blue underlined text.",
    "9446": "row 14, column 00, with plain cyan text.",
    "94c7": "row 14, column 00, with cyan underlined text.",
    "94c8": "row 14, column 00, with plain red text.",
    "9449": "row 14, column 00, with red underlined text.",
    "944a": "row 14, column 00, with plain yellow text.",
    "94cb": "row 14, column 00, with yellow underlined text.",
    "944c": "row 14, column 00, with plain magenta text.",
    "94cd": "row 14, column 00, with magenta underlined text.",
    "94ce": "row 14, column 00, with white italicized text.",
    "944f": "row 14, column 00, with white underlined italicized text.",
    "94d0": "row 14, column 00, with plain white text.",
    "9451": "row 14, column 00, with white underlined text.",
    "9452": "row 14, column 04, with plain white text.",
    "94d3": "row 14, column 04, with white underlined text.",
    "9454": "row 14, column 08, with plain white text.",
    "94d5": "row 14, column 08, with white underlined text.",
    "94d6": "row 14, column 12, with plain white text.",
    "9457": "row 14, column 12, with white underlined text.",
    "9458": "row 14, column 16, with plain white text.",
    "94d9": "row 14, column 16, with white underlined text.",
    "94da": "row 14, column 20, with plain white text.",
    "945b": "row 14, column 20, with white underlined text.",
    "94dc": "row 14, column 24, with plain white text.",
    "945d": "row 14, column 24, with white underlined text.",
    "945e": "row 14, column 28, with plain white text.",
    "94df": "row 14, column 28, with white underlined text.",
    "94e0": "row 15, column 00, with plain white text.",
    "9461": "row 15, column 00, with white underlined text.",
    "9462": "row 15, column 00, with plain green text.",
    "94e3": "row 15, column 00, with green underlined text.",
    "9464": "row 15, column 00, with plain blue text.",
    "94e5": "row 15, column 00, with blue underlined text.",
    "94e6": "row 15, column 00, with plain cyan text.",
    "9467": "row 15, column 00, with cyan underlined text.",
    "9468": "row 15, column 00, with plain red text.",
    "94e9": "row 15, column 00, with red underlined text.",
    "94ea": "row 15, column 00, with plain yellow text.",
    "946b": "row 15, column 00, with yellow underlined text.",
    "94ec": "row 15, column 00, with plain magenta text.",
    "946d": "row 15, column 00, with magenta underlined text.",
    "946e": "row 15, column 00, with white italicized text.",
    "94ef": "row 15, column 00, with white underlined italicized text.",
    "9470": "row 15, column 00, with plain white text.",
    "94f1": "row 15, column 00, with white underlined text.",
    "94f2": "row 15, column 04, with plain white text.",
    "9473": "row 15, column 04, with white underlined text.",
    "94f4": "row 15, column 08, with plain white text.",
    "9475": "row 15, column 08, with white underlined text.",
    "9476": "row 15, column 12, with plain white text.",
    "94f7": "row 15, column 12, with white underlined text.",
    "94f8": "row 15, column 16, with plain white text.",
    "9479": "row 15, column 16, with white underlined text.",
    "947a": "row 15, column 20, with plain white text.",
    "94fb": "row 15, column 20, with white underlined text.",
    "94fc": "row 15, column 24, with plain white text.",
    "94fd": "row 15, column 24, with white underlined text.",
    "94fe": "row 15, column 28, with plain white text.",
    "947f": "row 15, column 28, with white underlined text.",
    "97a1": "Tab Offset 1 column",
    "97a2": "Tab Offset 2 columns",
    "9723": "Tab Offset 3 columns",
    "94a1": "BackSpace",
    "94a4": "Delete to End of Row",
    "94ad": "Carriage Return",
    "1020": "Background White",
    "10a1": "Background Semi-Transparent White",
    "10a2": "Background Green",
    "1023": "Background Semi-Transparent Green",
    "10a4": "Background Blue",
    "1025": "Background Semi-Transparent Blue",
    "1026": "Background Cyan",
    "10a7": "Background Semi-Transparent Cyan",
    "10a8": "Background Red",
    "1029": "Background Semi-Transparent Red",
    "102a": "Background Yellow",
    "10ab": "Background Semi-Transparent Yellow",
    "102c": "Background Magenta",
    "10ad": "Background Semi-Transparent Magenta",
    "10ae": "Background Black",
    "102f": "Background Semi-Transparent Black",
    "97ad": "Background Transparent",
    "97a4": "Standard Character Set",
    "9725": "Double-Size Character Set",
    "9726": "First Private Character Set",
    "97a7": "Second Private Character Set",
    "97a8": "People`s Republic of China Character Set",
    "9729": "Korean Standard Character Set",
    "972a": "First Registered Character Set",
    "9120": "White",
    "91a1": "White Underline",
    "91a2": "Green",
    "9123": "Green Underline",
    "91a4": "Blue",
    "9125": "Blue Underline",
    "9126": "Cyan",
    "91a7": "Cyan Underline",
    "91a8": "Red",
    "9129": "Red Underline",
    "912a": "Yellow",
    "91ab": "Yellow Underline",
    "912c": "Magenta",
    "91ad": "Magenta Underline",
    "97ae": "Black",
    "972f": "Black Underline",
    "91ae": "Italics",
    "912f": "Italics Underline",
    "94a8": "Flash ON",
    "9423": "Alarm Off",
    "94a2": "Alarm On"
}


def translate_scc(scc_content, brackets='[]'):
    """
    Replaces hexadecimal words with their meaning

    In order to make SCC files more human readable and easier to debug,
    this function is used to replace command codes with their labels and
    character bytes with their actual characters

    :param scc_content: SCC captions to be translated
    :type scc_content: str
    :param brackets: Brackets to group the translated content of a command
    :type brackets: str
    :return: Translated SCC captions
    :rtype: str
    """
    opening_bracket, closing_bracket = brackets if brackets else ('', '')
    scc_elements = set(scc_content.split())
    for elem in scc_elements:
        name = COMMAND_LABELS.get(elem)
        # If a 2 byte command was not found, try retrieving 1 byte characters
        if not name:
            char1 = ALL_CHARACTERS.get(elem[:2])
            char2 = ALL_CHARACTERS.get(elem[2:])
            if char1 is not None and char2 is not None:
                name = f"{char1}{char2}"
        if name:
            scc_content = scc_content.replace(
                elem, f"{opening_bracket}{name}{closing_bracket}")
    return scc_content



import re

import pytest
from bs4 import BeautifulSoup

from pycaption.exceptions import InvalidInputError


class ReaderTestingMixIn:
    """
    Provide test case capabilities for asserting common Reader functionalities.
    """

    def assert_positive_answer_for_detection(self, matching_sample):
        assert self.reader.detect(matching_sample) is True

    def assert_negative_answer_for_detection(self, different_sample):
        assert self.reader.detect(different_sample) is False

    def test_reader_only_supports_unicode_input(self):
        with pytest.raises(InvalidInputError) as exc_info:
            self.reader.read(b'')
        assert exc_info.value.args[0] == 'The content is not a unicode string.'


class WebVTTTestingMixIn:
    """
    Provide specialized test case capabilities for asserting on WebVTT content.
    """

    def _extract_webvtt_captions(self, content):
        return tuple(line.strip() for line in content.splitlines())

    def assert_webvtt_equals(self, first, second):
        """
        Assert that two WebVTT contents are equal.
        """
        first_items = self._extract_webvtt_captions(first)
        second_items = self._extract_webvtt_captions(second)

        assert first_items == second_items


class SRTTestingMixIn:
    """
    Provide specialized test case capabilities for asserting on SRT content.
    """

    def _extract_srt_captions(self, content):
        return tuple(line.strip() for line in content.splitlines())

    def assert_srt_equals(self, first, second):
        """
        Assert that two SRT contents are equal.
        """
        first_items = self._extract_srt_captions(first)
        second_items = self._extract_srt_captions(second)

        assert first_items == second_items


class CaptionSetTestingMixIn:
    def assert_captionset_almost_equals(self, first, second,
                                        tolerance_microseconds):
        """
        Assert that two caption sets have equal text except for newlines,
        and differences in timing that are less than tolerance_microseconds.
        """

        captions_1 = first.get_captions(list(first.get_languages())[0])
        captions_2 = second.get_captions(list(first.get_languages())[0])

        def get_text_for_caption(caption):
            text = caption.get_text()
            text = re.sub(r'\s+', ' ', text)

            return text

        text_1 = [get_text_for_caption(caption) for caption in captions_1]
        text_2 = [get_text_for_caption(caption) for caption in captions_2]

        def close_enough(ts1, ts2):
            return abs(ts1 - ts2) < tolerance_microseconds

        start_differences = [
            (caption_1.start, caption_2.start)
            for caption_1, caption_2 in zip(captions_1, captions_2)
            if not close_enough(caption_1.start, caption_2.start)
        ]

        end_differences = [
            (caption_1.end, caption_2.end)
            for caption_1, caption_2 in zip(captions_1, captions_2)
            if not close_enough(caption_1.end, caption_2.end)
        ]

        assert text_1 == text_2
        assert start_differences == []
        assert end_differences == []


class DFXPTestingMixIn:
    """
    Provide specialized test case capabilities for asserting on DFXP content.
    """

    def _remove_styling(self, soup):
        for style in soup('styling'):
            style.clear()

        for paragraph in soup('p'):
            if 'style' in paragraph.attrs:
                del paragraph.attrs['style']

    def _remove_spans(self, soup):
        for span in soup('span'):
            span.unwrap()

    def _trim_text(self, soup):
        for paragraph in soup('p'):
            paragraph.string = paragraph.text.strip()

    def assert_dfxp_equals(self, first, second,
                           ignore_styling=False,
                           ignore_spans=False):
        first_soup = BeautifulSoup(first, 'lxml')
        second_soup = BeautifulSoup(second, 'lxml')

        if ignore_styling:
            self._remove_styling(first_soup)
            self._remove_styling(second_soup)

        if ignore_spans:
            self._remove_spans(first_soup)
            self._remove_spans(second_soup)

        self._trim_text(first_soup)
        self._trim_text(second_soup)

        assert first_soup == second_soup


class SAMITestingMixIn:
    """
    Provide specialized test case capabilities for asserting on SAMI content.
    """

    def _extract_sami_captions(self, soup):
        return tuple(
            (caption.attrs['start'], caption.p.text.strip())
            for caption in soup.select('sync')
        )

    def assert_sami_captions_equal(self, first, second):
        first_soup = BeautifulSoup(first, 'lxml')
        second_soup = BeautifulSoup(second, 'lxml')

        first_items = self._extract_sami_captions(first_soup)
        second_items = self._extract_sami_captions(second_soup)

        assert first_items == second_items


class MicroDVDTestingMixIn:
    """
    Provide specialized test case capabilities for asserting on MicroDVD content.
    """  # noqa

    def _extract_micro_dvd_captions(self, content):
        return tuple(line.strip() for line in content.splitlines())

    def assert_microdvd_equals(self, first, second):
        """
        Assert that two MicroDVD contents are equal.
        """
        first_items = self._extract_micro_dvd_captions(first)
        second_items = self._extract_micro_dvd_captions(second)

        assert first_items == second_items



import re

from pycaption import (
    DFXPReader, SAMIReader, SRTReader, SRTWriter, WebVTTReader,
)

from tests.mixins import SRTTestingMixIn


class TestDFXPtoSRT(SRTTestingMixIn):
    def setup_class(self):
        self.reader = DFXPReader()
        self.writer = SRTWriter()

    def test_dfxp_to_srt_conversion(self, sample_srt, sample_dfxp):
        caption_set = self.reader.read(sample_dfxp)
        results = self.writer.write(caption_set)

        assert isinstance(results, str)
        self.assert_srt_equals(sample_srt, results)

    def test_dfxp_empty_cue_to_srt(self, sample_srt_empty_cue_output,
                                   sample_dfxp_empty_cue):
        caption_set = self.reader.read(sample_dfxp_empty_cue)
        results = self.writer.write(caption_set)

        self.assert_srt_equals(sample_srt_empty_cue_output, results)


class TestSAMItoSRT(SRTTestingMixIn):
    def test_sami_to_srt_conversion(self, sample_srt, sample_sami):
        caption_set = SAMIReader().read(sample_sami)
        results = SRTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_srt_equals(sample_srt, results)


class TestSRTtoSRT(SRTTestingMixIn):
    def setup_class(self):
        self.reader = SRTReader()
        self.writer = SRTWriter()

    def test_srt_to_srt_conversion(self, sample_srt):
        caption_set = self.reader.read(sample_srt)
        results = self.writer.write(caption_set)

        assert isinstance(results, str)
        self.assert_srt_equals(sample_srt, results)

    def test_multiple_lines_for_one_sentence(self, samples_srt_same_time):
        caption_set = self.reader.read(samples_srt_same_time)
        results = self.writer.write(caption_set)
        sentences = re.split(r"\n\d\n", results)

        assert 3 == len(sentences)
        assert 4 == len(sentences[0].splitlines())


class TestWebVTTtoSRT(SRTTestingMixIn):
    def test_webvtt_to_srt_conversion(self, sample_srt, sample_webvtt):
        caption_set = WebVTTReader().read(sample_webvtt)
        results = SRTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_srt_equals(sample_srt, results)



from copy import deepcopy
from bs4 import BeautifulSoup

from pycaption.dfxp.base import _create_internal_alignment
from pycaption.dfxp import (
    SinglePositioningDFXPWriter, DFXPReader, DFXP_DEFAULT_REGION,
    DFXP_DEFAULT_REGION_ID, LegacyDFXPWriter,
)
from pycaption.geometry import (
    HorizontalAlignmentEnum, VerticalAlignmentEnum, Layout, Alignment,
)


class TestSinglePositioningDFXPWRiter:
    def test_only_the_default_region_is_created(
            self, sample_dfxp_to_render_with_only_default_positioning_input):
        caption_set = DFXPReader().read(
            sample_dfxp_to_render_with_only_default_positioning_input
        )

        dfxp = SinglePositioningDFXPWriter().write(caption_set)
        layout = BeautifulSoup(dfxp, features='html.parser').findChild('layout')

        assert len(layout.findChildren('region')) == 1

    def test_only_the_default_region_is_referenced(
            self, sample_dfxp_to_render_with_only_default_positioning_input):
        caption_set = DFXPReader().read(
            sample_dfxp_to_render_with_only_default_positioning_input
        )

        dfxp = SinglePositioningDFXPWriter().write(caption_set)

        soup = BeautifulSoup(dfxp, features='html.parser')

        for elem in soup.findAll():
            if 'region' in elem.attrs:
                assert elem['region'] == DFXP_DEFAULT_REGION_ID

    def test_only_the_custom_region_is_created(
            self, sample_dfxp_to_render_with_only_default_positioning_input):
        caption_set = DFXPReader().read(
            sample_dfxp_to_render_with_only_default_positioning_input
        )

        new_region = Layout(
            alignment=Alignment(
                HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP
            )
        )

        dfxp = SinglePositioningDFXPWriter(new_region).write(caption_set)
        # Using a different parser, because this preserves letter case
        # The output file is ok, but when parsing it, the "regular" parses
        # loses letter case.
        layout = BeautifulSoup(dfxp, features='xml').findChild('layout')

        region = layout.findChild('region')
        text_align = region['tts:textAlign']
        display_align = region['tts:displayAlign']

        internal_alignment = _create_internal_alignment(
            text_align, display_align
        )

        assert len(layout.findChildren('region')) == 1
        assert internal_alignment.horizontal == HorizontalAlignmentEnum.LEFT
        assert internal_alignment.vertical == VerticalAlignmentEnum.TOP

    def test_only_the_specified_custom_attributes_are_created_for_the_region(
        self, sample_dfxp_to_render_with_only_default_positioning_input
    ):
        caption_set = DFXPReader().read(
            sample_dfxp_to_render_with_only_default_positioning_input
        )

        new_region = Layout(
            alignment=Alignment(
                HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP
            )
        )

        dfxp = SinglePositioningDFXPWriter(new_region).write(caption_set)

        region = BeautifulSoup(dfxp).find('region')

        assert 'xml:id' in region.attrs
        assert region.attrs['xml:id'] != DFXP_DEFAULT_REGION_ID
        assert len(region.attrs) == 3

    def test_only_the_custom_region_is_referenced(
            self, sample_dfxp_to_render_with_only_default_positioning_input):
        caption_set = DFXPReader().read(
            sample_dfxp_to_render_with_only_default_positioning_input
        )

        # it's easier to copy this than create a new one
        new_region = deepcopy(DFXP_DEFAULT_REGION)
        new_region.alignment.horizontal = HorizontalAlignmentEnum.LEFT
        new_region.alignment.vertical = VerticalAlignmentEnum.TOP

        dfxp = SinglePositioningDFXPWriter(new_region).write(caption_set)

        soup = BeautifulSoup(dfxp, features='html.parser')

        # get the region_id created, and see it's the one referenced
        created_region_id = soup.find('region')['xml:id']

        referenced_region_ids = set()

        for elem in soup.findAll():
            if 'region' in elem.attrs:
                referenced_region_ids.add(elem.attrs['region'])

        assert len(referenced_region_ids) == 1
        assert referenced_region_ids.pop() == created_region_id

    def test_styles_dont_contain_text_align_attribute(
            self, sample_dfxp_to_render_with_only_default_positioning_input):
        caption_set = DFXPReader().read(
            sample_dfxp_to_render_with_only_default_positioning_input
        )

        result = SinglePositioningDFXPWriter().write(caption_set)

        caption_set = DFXPReader().read(result)

        for _, style in caption_set.get_styles():
            assert 'text-align' not in style


class TestLegacyDFXPWriter:
    def test_default_style_is_written_to_output_file(
            self, sample_dfxp_with_templated_style):
        caption_set = DFXPReader(read_invalid_positioning=True).read(
            sample_dfxp_with_templated_style.format(
                style_name="foxy_the_squirrel"))

        result = LegacyDFXPWriter().write(caption_set)

        assert result.count('foxy_the_squirrel') == 2



import pytest

from pycaption.base import CaptionList, Caption


class TestCaption:
    def setup_method(self):
        self.caption = Caption(0, 999999999999, ['test'])

    def test_format_start(self):
        assert self.caption.format_start() == '00:00:00.000'

    def test_format_end(self):
        assert self.caption.format_end() == '13:46:39.999'


class TestCaptionList:
    def setup_method(self):
        self.layout_info = "My Layout"
        self.caps = CaptionList([1, 2, 3], layout_info=self.layout_info)

    def test_splice(self):
        newcaps = self.caps[1:]

        assert isinstance(newcaps, CaptionList)
        assert newcaps.layout_info == self.layout_info

    def test_mul(self):
        newcaps = self.caps * 2

        assert isinstance(newcaps, CaptionList)
        assert newcaps.layout_info == self.layout_info

    def test_rmul(self):
        newcaps = 2 * self.caps

        assert isinstance(newcaps, CaptionList)
        assert newcaps.layout_info == self.layout_info

    def test_add_list_to_caption_list(self):
        newcaps = self.caps + [9, 8, 7]

        assert isinstance(newcaps, CaptionList)
        assert newcaps.layout_info == self.layout_info

    def test_add_two_caption_lists(self):
        newcaps = self.caps + CaptionList([4], layout_info=None)

        assert isinstance(newcaps, CaptionList)
        assert newcaps.layout_info == self.layout_info

        newcaps = self.caps + CaptionList([4], layout_info=self.layout_info)

        assert isinstance(newcaps, CaptionList)
        assert newcaps.layout_info == self.layout_info

        with pytest.raises(ValueError):
            newcaps = self.caps + CaptionList([4], layout_info="Other Layout")



import pytest

from pycaption import MicroDVDReader, CaptionReadNoCaptions
from pycaption.exceptions import CaptionReadSyntaxError, CaptionReadTimingError
from pycaption.base import DEFAULT_LANGUAGE_CODE
from tests.mixins import ReaderTestingMixIn


class TestMicroDVDReader(ReaderTestingMixIn):
    def setup_class(self):
        self.reader = MicroDVDReader()

    def test_positive_answer_for_detection(self, sample_microdvd):
        super().assert_positive_answer_for_detection(sample_microdvd)

    @pytest.mark.parametrize('different_sample', [
        pytest.lazy_fixture('sample_dfxp'),
        pytest.lazy_fixture('sample_sami'),
        pytest.lazy_fixture('sample_scc_pop_on'),
        pytest.lazy_fixture('sample_srt'),
        pytest.lazy_fixture('sample_webvtt')
    ])
    def test_negative_answer_for_detection(self, different_sample):
        super().assert_negative_answer_for_detection(different_sample)

    def test_caption_length(self, sample_microdvd):
        captions = MicroDVDReader().read(sample_microdvd)

        assert 7 == len(captions.get_captions(DEFAULT_LANGUAGE_CODE))

    def test_proper_timestamps(self, sample_microdvd):
        captions = MicroDVDReader().read(sample_microdvd)
        paragraph = captions.get_captions(DEFAULT_LANGUAGE_CODE)[2]

        # due to lossy nature of microsec -> frame# we check that
        # conversion is within a second of expected value
        # (fyi: timestamps in examples/ and tests/fixtures/ differ)
        assert abs(17350000 - paragraph.start) < 10 ** 6
        assert abs(18752000 - paragraph.end) < 10 ** 6

    def test_empty_file(self, sample_microdvd_empty):
        with pytest.raises(CaptionReadNoCaptions):
            MicroDVDReader().read(sample_microdvd_empty)

    def test_invalid_format(self, sample_microdvd_invalid_format):
        with pytest.raises(CaptionReadSyntaxError):
            MicroDVDReader().read(sample_microdvd_invalid_format)

    def test_no_fps_provided(self, missing_fps_sample_microdvd):
        with pytest.raises(CaptionReadTimingError):
            MicroDVDReader().read(missing_fps_sample_microdvd)
